from jinja2 import Environment, FileSystemLoader
import yaml
import os
import sys
import stringcase
import shutil
import uncrustify
import clang_format_runner as clang_format_runner
import generation_dict_tools as tools
from pathlib import Path

def _get_generation_dict(manifest_data) -> dict:
    generation_dict = {}

    cmakelists_dict = {}

    application_name = manifest_data.get("global", {}).get("application_name", "example")
    application_cpp_domain_name = manifest_data.get("global", {}).get("application_cpp_domain_name", "example")

    # CMakeLists.txt

    cmakelists_dict["application_name"] = application_name

    front_ends = manifest_data.get("front_ends", {})
    front_end_dict = {}

    front_end_dict["qt_widgets"] = front_ends.get("qt_widgets", {})
    if front_end_dict["qt_widgets"]:
        assert front_end_dict["qt_widgets"]["folder_path"], "qt_widgets folder_path is not set in the manifest file"
    folder_path = front_end_dict["qt_widgets"]["folder_path"]

    cmakelists_dict["folder_path"] = folder_path
    generation_dict["cmakelists"] = cmakelists_dict

    # main.cpp

    main_cpp_dict = {}
    main_cpp_dict["folder_path"] = folder_path
    main_cpp_dict["application_name"] = application_name
    main_cpp_dict["application_cpp_domain_name"] = application_cpp_domain_name
    main_cpp_dict["organisation_domain"] = manifest_data.get("global", {}).get("organisation", {}).get("domain", "example.com")
    main_cpp_dict["organisation_name"] = manifest_data.get("global", {}).get("organisation", {}).get("name", "example")

    generation_dict["main_cpp"] = main_cpp_dict

    # mainwindow files
    mainwindow_dict = {}
    mainwindow_dict["folder_path"] = folder_path
    mainwindow_dict["application_name"] = application_name
    mainwindow_dict["application_cpp_domain_name"] = application_cpp_domain_name

    generation_dict["mainwindow"] = mainwindow_dict

    return generation_dict


def _generate_cmakelists_file(
    root_path: str, cmakelists_dict: dict, files_to_be_generated: dict[str, bool]
):
    cmakelists_file = os.path.join(cmakelists_dict["folder_path"], "CMakeLists.txt")
    
    # Create the jinja2 environment
    template_env = Environment(loader=FileSystemLoader("templates/UIs/qt_widgets"))
    # Load the template
    cmakelists_template = template_env.get_template("CMakeLists.txt.jinja2")

    # generate the real cmakelists file if in the files_to_be_generated dict the value is True
    if not files_to_be_generated.get(cmakelists_file, False):
        return

    output_file = os.path.join(root_path, cmakelists_file)

    # Render the template
    output = cmakelists_template.render(
        application_name=cmakelists_dict["application_name"],
        application_spinal_name=stringcase.spinalcase(cmakelists_dict["application_name"]),
        application_pascal_name=stringcase.pascalcase(cmakelists_dict["application_name"]),
    )

    # Create the directory if it does not exist
    os.makedirs(os.path.dirname(output_file), exist_ok=True)

    # Write the output to the file
    with open(output_file, "w") as fh:
        fh.write(output)

    print(f"Successfully wrote file {output_file}")

def _generate_main_cpp_file(
    root_path: str,
    main_cpp_dict: dict,
    files_to_be_generated: dict[str, bool]
):
    main_cpp_file = os.path.join(main_cpp_dict["folder_path"], "main.cpp")
    
    # Create the jinja2 environment
    template_env = Environment(loader=FileSystemLoader("templates/UIs/qt_widgets"))
    # Load the template
    main_cpp_template = template_env.get_template("main.cpp.jinja2")

    # generate the real main.cpp file if in the files_to_be_generated dict the value is True
    if not files_to_be_generated.get(main_cpp_file, False):
        return

    output_file = os.path.join(root_path, main_cpp_file)

    # Render the template
    output = main_cpp_template.render(
        application_name=main_cpp_dict["application_name"],
        application_cpp_domain_name=main_cpp_dict["application_cpp_domain_name"],
        organisation_domain=main_cpp_dict["organisation_domain"],
        organisation_name=main_cpp_dict["organisation_name"],
    )

    # Create the directory if it does not exist
    os.makedirs(os.path.dirname(output_file), exist_ok=True)

    # Write the output to the file
    with open(output_file, "w") as fh:
        fh.write(output)

    print(f"Successfully wrote file {output_file}")

def _generate_mainwindow_files(
    root_path: str,
    mainwindow_dict: dict,
    files_to_be_generated: dict[str, bool]
):
    mainwindow_h_file = os.path.join(mainwindow_dict["folder_path"], "mainwindow.h")
    mainwindow_cpp_file = os.path.join(mainwindow_dict["folder_path"], "mainwindow.cpp")
    mainwindow_ui_file = os.path.join(mainwindow_dict["folder_path"], "mainwindow.ui")
    
    # Create the jinja2 environment
    template_env = Environment(loader=FileSystemLoader("templates/UIs/qt_widgets"))
    # Load the template
    mainwindow_h_template = template_env.get_template("mainwindow.h.jinja2")
    mainwindow_cpp_template = template_env.get_template("mainwindow.cpp.jinja2")
    mainwindow_ui_template = template_env.get_template("mainwindow.ui.jinja2")

    # generate the real mainwindow files if in the files_to_be_generated dict the value is True
    if not files_to_be_generated.get(mainwindow_h_file, False):
        return
    if not files_to_be_generated.get(mainwindow_cpp_file, False):
        return
    if not files_to_be_generated.get(mainwindow_ui_file, False):
        return

    output_h_file = os.path.join(root_path, mainwindow_h_file)
    output_cpp_file = os.path.join(root_path, mainwindow_cpp_file)
    output_ui_file = os.path.join(root_path, mainwindow_ui_file)

    # Render the template
    output_h = mainwindow_h_template.render(
        application_name=mainwindow_dict["application_name"],
        application_pascal_name=stringcase.pascalcase(mainwindow_dict["application_name"]),
        application_cpp_domain_name=mainwindow_dict["application_cpp_domain_name"],
    )
    output_cpp = mainwindow_cpp_template.render(
        application_name=mainwindow_dict["application_name"],
        application_pascal_name=stringcase.pascalcase(mainwindow_dict["application_name"]),
        application_cpp_domain_name=mainwindow_dict["application_cpp_domain_name"],
    )
    output_ui = mainwindow_ui_template.render(
        application_name=mainwindow_dict["application_name"],
        application_pascal_name=stringcase.pascalcase(mainwindow_dict["application_name"]),
    )

    # Create the directory if it does not exist
    os.makedirs(os.path.dirname(output_h_file), exist_ok=True)
    os.makedirs(os.path.dirname(output_cpp_file), exist_ok=True)
    os.makedirs(os.path.dirname(output_ui_file), exist_ok=True)

    # Write the output to the file
    with open(output_h_file, "w") as fh:
        fh.write(output_h)
    with open(output_cpp_file, "w") as fh:
        fh.write(output_cpp)
    with open(output_ui_file, "w") as fh:
        fh.write(output_ui)

    print(f"Successfully wrote file {output_h_file}")
    print(f"Successfully wrote file {output_cpp_file}")
    print(f"Successfully wrote file {output_ui_file}")

def generate_qt_widgets_files(
    root_path: str,
    manifest_file: str,
    files_to_be_generated: dict[str, bool] = {},
    uncrustify_config_file: str = None,
):

    with open(manifest_file, "r") as stream:
        try:
            manifest_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
            return

    generation_dict = _get_generation_dict(manifest_data)


    # generate the CMakeLists.txt file
    _generate_cmakelists_file(root_path, generation_dict["cmakelists"], files_to_be_generated)

    # generate the main.cpp file
    _generate_main_cpp_file(root_path, generation_dict["main_cpp"], files_to_be_generated)

    # generate the mainwindow files
    _generate_mainwindow_files(root_path, generation_dict["mainwindow"], files_to_be_generated)



def get_files_to_be_generated(
    manifest_file: str, files_to_be_generated: dict[str, bool] = {}
) -> list[str]:
    """
    Get the list of files that need to be generated based on the manifest file
    """
    # Read the manifest file
    with open(manifest_file, "r") as stream:
        try:
            manifest_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
            return
        
    generation_dict = _get_generation_dict(manifest_data)

    folder_path = generation_dict["cmakelists"]["folder_path"]

    files = []
    files.append(os.path.join(folder_path, "CMakeLists.txt"))
    files.append(os.path.join(folder_path, "main.cpp"))
    files.append(os.path.join(folder_path, "mainwindow.h"))
    files.append(os.path.join(folder_path, "mainwindow.cpp"))
    files.append(os.path.join(folder_path, "mainwindow.ui"))

    # strip from files if the value in files_to_be_generated is False
    if files_to_be_generated:
        for path, generate in files_to_be_generated.items():
            if not generate and path in files:
                files.remove(path)

    return files


# generate the files into the preview folder
def preview_qt_widgets_files(
    root_path: str,
    manifest_file: str,
    files_to_be_generated: dict[str, bool] = {},
    uncrustify_config_file: str = None,
):
    manifest_preview_file = "temp/manifest_preview.yaml"


    # make a copy of the manifest file into temp/manifest_preview.yaml
    shutil.copy(manifest_file, manifest_preview_file)

    # modify the manifest file to generate the files into the preview folder
    with open(manifest_preview_file, "r") as fh:
        manifest = yaml.safe_load(fh)

    # write the modified manifest file
    with open(manifest_preview_file, "w") as fh:
        yaml.dump(manifest, fh)

    root_path = os.path.join(root_path, "qleany_preview")

    # remove .. from the path
    if files_to_be_generated:
        preview_files_to_be_generated = {}
        for path, value in files_to_be_generated.items():
            preview_files_to_be_generated[path.replace("..", "")] = value

        generate_qt_widgets_files(
            root_path,
            manifest_preview_file,
            preview_files_to_be_generated,
            uncrustify_config_file,
        )

    else:
        generate_qt_widgets_files(root_path, manifest_preview_file, {}, uncrustify_config_file)
        

def is_enabled(manifest_file: str) -> bool:
    with open(manifest_file, "r") as stream:
        try:
            manifest_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
            return False

    front_ends = manifest_data.get("front_ends", {})
    if not front_ends:
        return False

    front_end_dict = {}

    front_end_dict["qt_widgets"] = front_ends.get("qt_widgets", {})
    if front_end_dict["qt_widgets"]:
        assert front_end_dict["qt_widgets"]["folder_path"], "qt_widgets folder_path is not set in the manifest file"
        return True
    else:
        return False

# Main execution
if __name__ == "__main__":
    full_path = Path(__file__).resolve().parent

    # Add the current directory to the path so that we can import the generated files
    sys.path.append(full_path)

    # Set the current directory to the generator directory
    os.chdir(full_path)

    if len(sys.argv) > 1:
        manifest_arg = sys.argv[1]
        if manifest_arg.endswith(".yaml") or manifest_arg.endswith(".yml"):
            manifest_file = manifest_arg
            root_path = Path(manifest_file).parent

            if len(sys.argv) > 2 and sys.argv[2] == "--preview":
                preview_qt_widgets_files(root_path, manifest_file)
            else:
                generate_qt_widgets_files(root_path, manifest_file)
        else:
            print("Error: Manifest file must be named 'qleany.yaml' or 'qleany.yml'")
    else:
        print("Error: Please provide the manifest file as an argument")



import ruamel.yaml
import json
import jsonschema


def validate_manifest(yaml_file: str) -> str:

    try:
        # load yaml file
        with open(yaml_file, "r") as file:
            yaml = ruamel.yaml.YAML(typ="safe", pure=True)
            yaml_manifest = yaml.load(file)

        # dump yaml to json
        json_manifest = json.loads(json.dumps(yaml_manifest))

        with open("manifest_schema.json", "r") as file:
            schema = json.load(file)

        jsonschema.validate(json_manifest, schema)

    except Exception as e:
        print(f"Error: {e}")
        return str(e)

    return None



from PySide6.QtWidgets import (
    QMainWindow,
    QApplication,
    QVBoxLayout,
    QTreeView,
    QPushButton,
    QPlainTextEdit,
    QSplitter,
    QMenu,
    QHBoxLayout,
    QWidget,
    QGroupBox,
    QMessageBox,
    QCheckBox,
    QListView,
    QTextEdit,
    QSizePolicy,
    QLabel,
    QDialog,
    QDialogButtonBox,
    QFileDialog,
    QProgressDialog,
)
from PySide6.QtCore import (
    Qt,
    QAbstractItemModel,
    QModelIndex,
    QCoreApplication,
    QSettings,
    QFileSystemWatcher,
    QTimer,
)
from PySide6.QtGui import QStandardItemModel, QStandardItem
import sys
import os
import yaml
import shutil
from pathlib import Path

full_path = Path(__file__).resolve().parent
full_path = f"{full_path}"
# add the current directory to the path so that we can import the generated files
sys.path.append(full_path)

# set the current directory to the generator directory
os.chdir(full_path)

import __version__
import entities_generator
import dto_generator
import repositories_generator
import cqrs_generator
import controller_generator
import application_generator
import qml_imports_generator
import entity_relationship_viewer
import presenter_generator
import root_generator
import qt_widgets_generator
import qt_quick_generator
import kf6_kirigami_generator

# this little application is a GUI for the generator

# This little application is a GUI for the generator. It allows you to select which file to generate in the middle view.
# It also allows you to preview the files in the "preview" folder by the python script before generating them properly.
# This GUI uses the qleany.yaml file placed at the root of your project and you can cherry-pick which files to generate.
# The states of the checkboxes are saved in the settings.yaml file.
# The qleany.yaml file is not modified by this UI.
# The manifest_temp.yaml file is a modified copy of the qleany.yaml file, but exists only for argument passing to the generator scripts.
# Any modification of the qleany.yaml file will be reflected in the UI.


class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)

        self.settings = QSettings()

        self.manifest_file = self.settings.value("last_selected_manifest_path", "")
        self.temp_manifest_file = "temp/manifest_temp.yaml"
        self.settings_file = "temp/settings.yaml"

        # qleany file watcher
        self.watcher = QFileSystemWatcher([self.manifest_file])
        self.watcher.fileChanged.connect(self.on_manifest_file_changed)

        self.timer = QTimer()
        self.timer.setInterval(1000)  # check every second
        self.timer.timeout.connect(self.check_manifest_file)

        self.uncrustify_config_file = "../uncrustify.cfg"

        # geometry

        self.setGeometry(100, 100, 800, 600)
        self.setWindowTitle(f"Qleany, the Qt Clean Architecture Generator, version {__version__.__version__}")

        # UI
        self.manifest_refreshed_displayed = False
        self.central_widget = QWidget()
        self.setCentralWidget(self.central_widget)
        self.central_layout = QVBoxLayout()
        self.central_widget.setLayout(self.central_layout)

        self.manifest_file_layout = QHBoxLayout()
        # set entity relationship window launch button

        self.entity_relationship_window_button = QPushButton(
            "Entity Relationship Viewer"
        )
        self.entity_relationship_window_button.clicked.connect(
            self.open_entity_relationship_window
        )
        self.manifest_file_layout.addWidget(self.entity_relationship_window_button)

        # set qleany.yml selector

        self.manifest_file_label = QLabel("Qleany YAML file:")
        self.manifest_file_layout.addWidget(self.manifest_file_label)
        self.manifest_file_text = QLabel(self.manifest_file)
        self.manifest_file_layout.addWidget(self.manifest_file_text)
        self.manifest_file_button = QPushButton("Select")
        self.manifest_file_button.clicked.connect(self.select_qleany_manifest_file)
        self.manifest_file_layout.addWidget(self.manifest_file_button)
        self.central_layout.addLayout(self.manifest_file_layout)
        self.central_layout.setStretch(0, 0)

        # set splitter for views
        self.splitter = QSplitter()
        self.central_layout.addWidget(self.splitter)
        self.central_layout.setStretch(1, 1)

        self.tree = QTreeView(self.splitter)
        self.tree.setContextMenuPolicy(Qt.CustomContextMenu)
        self.tree.customContextMenuRequested.connect(self.open_menu)

        self.file_list_view = CheckableFileListView(self.splitter)

        self.text_box = QPlainTextEdit(self.splitter)
        self.text_box.setReadOnly(True)

        self.tree.clicked.connect(self.on_tree_item_click)
        self.tree.setAlternatingRowColors(True)
        self.tree.setEditTriggers(QTreeView.NoEditTriggers)

        self.first_row_button_layout = QHBoxLayout()

        self.tools_group_box = QGroupBox()
        self.tools_group_box.setTitle("Tools")
        self.tools_layout = QVBoxLayout()
        self.tools_group_box.setLayout(self.tools_layout)

        # Expand All

        self.btn_expand_all = QPushButton("Expand All", self)
        self.btn_expand_all.clicked.connect(self.expand_all)
        self.tools_layout.addWidget(self.btn_expand_all)

        # Clear preview folder

        self.btn_clear_preview_folder = QPushButton("Clear Preview Folder", self)
        self.btn_clear_preview_folder.clicked.connect(self.clear_preview_folder)
        self.tools_layout.addWidget(self.btn_clear_preview_folder)

        # Refresh

        self.btn_refresh = QPushButton("Refresh", self)
        self.btn_refresh.clicked.connect(self.refresh)
        self.tools_layout.addWidget(self.btn_refresh)

        self.first_row_button_layout.addWidget(self.tools_group_box)

        # Generate root files

        self.generate_root_group_box = QGroupBox()
        self.generate_root_group_box.setTitle("Generate Root Files")
        self.generate_root_layout = QVBoxLayout()
        self.generate_root_group_box.setLayout(self.generate_root_layout)

        self.btn_list_root = QPushButton("List", self)
        self.btn_list_root.clicked.connect(self.list_root)
        self.generate_root_layout.addWidget(self.btn_list_root)

        self.btn_preview_root = QPushButton("Preview", self)
        self.btn_preview_root.clicked.connect(self.preview_root)
        self.generate_root_layout.addWidget(self.btn_preview_root)

        self.btn_generate_root = QPushButton("Generate", self)
        self.btn_generate_root.clicked.connect(self.generate_root)
        self.generate_root_layout.addWidget(self.btn_generate_root)

        self.first_row_button_layout.addWidget(self.generate_root_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_root.setEnabled(False)
        self.btn_generate_root.setEnabled(False)

        def enable_root_buttons():
            self.btn_preview_root.setEnabled(True)
            self.btn_generate_root.setEnabled(True)

        self.btn_list_root.clicked.connect(enable_root_buttons)

        # Generate Entities

        self.generate_entities_group_box = QGroupBox()
        self.generate_entities_group_box.setTitle("Generate Entities")
        self.generate_entities_layout = QVBoxLayout()
        self.generate_entities_group_box.setLayout(self.generate_entities_layout)

        self.btn_list_entities = QPushButton("List", self)
        self.btn_list_entities.clicked.connect(self.list_entities)
        self.generate_entities_layout.addWidget(self.btn_list_entities)

        self.btn_preview_entities = QPushButton("Preview", self)
        self.btn_preview_entities.clicked.connect(self.preview_entities)
        self.generate_entities_layout.addWidget(self.btn_preview_entities)

        self.btn_generate_entities = QPushButton("Generate", self)
        self.btn_generate_entities.clicked.connect(self.generate_entities)
        self.generate_entities_layout.addWidget(self.btn_generate_entities)

        self.first_row_button_layout.addWidget(self.generate_entities_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_entities.setEnabled(False)
        self.btn_generate_entities.setEnabled(False)

        def enable_entities_buttons():
            self.btn_preview_entities.setEnabled(True)
            self.btn_generate_entities.setEnabled(True)

        self.btn_list_entities.clicked.connect(enable_entities_buttons)

        # Generate DTOs

        self.generate_dtos_group_box = QGroupBox()
        self.generate_dtos_group_box.setTitle("Generate DTOs")
        self.generate_dtos_layout = QVBoxLayout()
        self.generate_dtos_group_box.setLayout(self.generate_dtos_layout)

        self.btn_list_dtos = QPushButton("List", self)
        self.btn_list_dtos.clicked.connect(self.list_dtos)
        self.generate_dtos_layout.addWidget(self.btn_list_dtos)

        self.btn_preview_dtos = QPushButton("Preview", self)
        self.btn_preview_dtos.clicked.connect(self.preview_dtos)
        self.generate_dtos_layout.addWidget(self.btn_preview_dtos)

        self.btn_generate_dtos = QPushButton("Generate", self)
        self.btn_generate_dtos.clicked.connect(self.generate_dtos)
        self.generate_dtos_layout.addWidget(self.btn_generate_dtos)

        self.first_row_button_layout.addWidget(self.generate_dtos_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_dtos.setEnabled(False)
        self.btn_generate_dtos.setEnabled(False)

        def enable_dtos_buttons():
            self.btn_preview_dtos.setEnabled(True)
            self.btn_generate_dtos.setEnabled(True)

        self.btn_list_dtos.clicked.connect(enable_dtos_buttons)

        # Generate Repositories

        self.generate_repositories_group_box = QGroupBox()
        self.generate_repositories_group_box.setTitle("Generate Repositories")
        self.generate_repositories_layout = QVBoxLayout()
        self.generate_repositories_group_box.setLayout(
            self.generate_repositories_layout
        )

        self.btn_list_repositories = QPushButton("List", self)
        self.btn_list_repositories.clicked.connect(self.list_repositories)
        self.generate_repositories_layout.addWidget(self.btn_list_repositories)

        self.btn_preview_repositories = QPushButton("Preview", self)
        self.btn_preview_repositories.clicked.connect(self.preview_repositories)
        self.generate_repositories_layout.addWidget(self.btn_preview_repositories)

        self.btn_generate_repositories = QPushButton("Generate", self)
        self.btn_generate_repositories.clicked.connect(self.generate_repositories)
        self.generate_repositories_layout.addWidget(self.btn_generate_repositories)

        self.first_row_button_layout.addWidget(self.generate_repositories_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_repositories.setEnabled(False)
        self.btn_generate_repositories.setEnabled(False)

        def enable_repositories_buttons():
            self.btn_preview_repositories.setEnabled(True)
            self.btn_generate_repositories.setEnabled(True)

        self.btn_list_repositories.clicked.connect(enable_repositories_buttons)

        # Generate CQRS

        self.generate_cqrs_group_box = QGroupBox()
        self.generate_cqrs_group_box.setTitle("Generate CQRS")
        self.generate_cqrs_layout = QVBoxLayout()
        self.generate_cqrs_group_box.setLayout(self.generate_cqrs_layout)

        self.btn_list_cqrs = QPushButton("List", self)
        self.btn_list_cqrs.clicked.connect(self.list_cqrs)
        self.generate_cqrs_layout.addWidget(self.btn_list_cqrs)

        self.btn_preview_cqrs = QPushButton("Preview", self)
        self.btn_preview_cqrs.clicked.connect(self.preview_cqrs)
        self.generate_cqrs_layout.addWidget(self.btn_preview_cqrs)

        self.btn_generate_cqrs = QPushButton("Generate", self)
        self.btn_generate_cqrs.clicked.connect(self.generate_cqrs)
        self.generate_cqrs_layout.addWidget(self.btn_generate_cqrs)

        self.first_row_button_layout.addWidget(self.generate_cqrs_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_cqrs.setEnabled(False)
        self.btn_generate_cqrs.setEnabled(False)

        def enable_cqrs_buttons():
            self.btn_preview_cqrs.setEnabled(True)
            self.btn_generate_cqrs.setEnabled(True)

        self.btn_list_cqrs.clicked.connect(enable_cqrs_buttons)

        # Generate application

        self.generate_application_group_box = QGroupBox()
        self.generate_application_group_box.setTitle("Generate Application")
        self.generate_application_layout = QVBoxLayout()
        self.generate_application_group_box.setLayout(self.generate_application_layout)

        self.btn_list_application = QPushButton("List", self)
        self.btn_list_application.clicked.connect(self.list_application)
        self.generate_application_layout.addWidget(self.btn_list_application)

        self.btn_preview_application = QPushButton("Preview", self)
        self.btn_preview_application.clicked.connect(self.preview_application)
        self.generate_application_layout.addWidget(self.btn_preview_application)

        self.btn_generate_application = QPushButton("Generate", self)
        self.btn_generate_application.clicked.connect(self.generate_application)
        self.generate_application_layout.addWidget(self.btn_generate_application)

        self.first_row_button_layout.addWidget(self.generate_application_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_application.setEnabled(False)
        self.btn_generate_application.setEnabled(False)

        def enable_application_buttons():
            self.btn_preview_application.setEnabled(True)
            self.btn_generate_application.setEnabled(True)

        self.btn_list_application.clicked.connect(enable_application_buttons)

        # Generate Controllers

        self.generate_controllers_group_box = QGroupBox()
        self.generate_controllers_group_box.setTitle("Generate Controllers")
        self.generate_controllers_layout = QVBoxLayout()
        self.generate_controllers_group_box.setLayout(self.generate_controllers_layout)

        self.btn_list_controllers = QPushButton("List", self)
        self.btn_list_controllers.clicked.connect(self.list_controllers)
        self.generate_controllers_layout.addWidget(self.btn_list_controllers)

        self.btn_preview_controllers = QPushButton("Preview", self)
        self.btn_preview_controllers.clicked.connect(self.preview_controllers)
        self.generate_controllers_layout.addWidget(self.btn_preview_controllers)

        self.btn_generate_controllers = QPushButton("Generate", self)
        self.btn_generate_controllers.clicked.connect(self.generate_controllers)
        self.generate_controllers_layout.addWidget(self.btn_generate_controllers)

        self.first_row_button_layout.addWidget(self.generate_controllers_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_controllers.setEnabled(False)
        self.btn_generate_controllers.setEnabled(False)

        def enable_controllers_buttons():
            self.btn_preview_controllers.setEnabled(True)
            self.btn_generate_controllers.setEnabled(True)

        self.btn_list_controllers.clicked.connect(enable_controllers_buttons)

        # Generate Presenters

        self.generate_presenters_group_box = QGroupBox()
        self.generate_presenters_group_box.setTitle("Generate Presenters")
        self.generate_presenters_layout = QVBoxLayout()
        self.generate_presenters_group_box.setLayout(self.generate_presenters_layout)

        self.btn_list_presenters = QPushButton("List", self)
        self.btn_list_presenters.clicked.connect(self.list_presenters)
        self.generate_presenters_layout.addWidget(self.btn_list_presenters)

        self.btn_preview_presenters = QPushButton("Preview", self)
        self.btn_preview_presenters.clicked.connect(self.preview_presenters)
        self.generate_presenters_layout.addWidget(self.btn_preview_presenters)

        self.btn_generate_presenters = QPushButton("Generate", self)
        self.btn_generate_presenters.clicked.connect(self.generate_presenters)
        self.generate_presenters_layout.addWidget(self.btn_generate_presenters)

        self.first_row_button_layout.addWidget(self.generate_presenters_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_presenters.setEnabled(False)
        self.btn_generate_presenters.setEnabled(False)

        def enable_presenters_buttons():
            self.btn_preview_presenters.setEnabled(True)
            self.btn_generate_presenters.setEnabled(True)

        self.btn_list_presenters.clicked.connect(enable_presenters_buttons)


        # second row of buttons
        self.second_row_button_layout = QHBoxLayout()

        # generate qt widgets ui

        self.generate_qt_widgets_ui_group_box = QGroupBox()
        self.generate_qt_widgets_ui_group_box.setTitle("Generate Qt Widgets UI")
        self.generate_qt_widgets_ui_layout = QVBoxLayout()
        self.generate_qt_widgets_ui_group_box.setLayout(self.generate_qt_widgets_ui_layout)

        self.btn_list_qt_widgets_ui = QPushButton("List", self)
        self.btn_list_qt_widgets_ui.clicked.connect(self.list_qt_widgets_ui)
        self.generate_qt_widgets_ui_layout.addWidget(self.btn_list_qt_widgets_ui)

        self.btn_preview_qt_widgets_ui = QPushButton("Preview", self)
        self.btn_preview_qt_widgets_ui.clicked.connect(self.preview_qt_widgets_ui)
        self.generate_qt_widgets_ui_layout.addWidget(self.btn_preview_qt_widgets_ui)

        self.btn_generate_qt_widgets_ui = QPushButton("Generate", self)
        self.btn_generate_qt_widgets_ui.clicked.connect(self.generate_qt_widgets_ui)
        self.generate_qt_widgets_ui_layout.addWidget(self.btn_generate_qt_widgets_ui)

        self.second_row_button_layout.addWidget(self.generate_qt_widgets_ui_group_box)

        # disable preview and generate buttons if list button is not clicked once

        self.btn_preview_qt_widgets_ui.setEnabled(False)
        self.btn_generate_qt_widgets_ui.setEnabled(False)

        def enable_qt_widgets_ui_buttons():
            self.btn_preview_qt_widgets_ui.setEnabled(True)
            self.btn_generate_qt_widgets_ui.setEnabled(True)

        self.btn_list_qt_widgets_ui.clicked.connect(enable_qt_widgets_ui_buttons)

        # generate qt quick ui

        self.generate_qt_quick_ui_group_box = QGroupBox()
        self.generate_qt_quick_ui_group_box.setTitle("Generate Qt Quick UI")
        self.generate_qt_quick_ui_layout = QVBoxLayout()
        self.generate_qt_quick_ui_group_box.setLayout(self.generate_qt_quick_ui_layout)

        self.btn_list_qt_quick_ui = QPushButton("List", self)
        self.btn_list_qt_quick_ui.clicked.connect(self.list_qt_quick_ui)
        self.generate_qt_quick_ui_layout.addWidget(self.btn_list_qt_quick_ui)

        self.btn_preview_qt_quick_ui = QPushButton("Preview", self)
        self.btn_preview_qt_quick_ui.clicked.connect(self.preview_qt_quick_ui)
        self.generate_qt_quick_ui_layout.addWidget(self.btn_preview_qt_quick_ui)

        self.btn_generate_qt_quick_ui = QPushButton("Generate", self)
        self.btn_generate_qt_quick_ui.clicked.connect(self.generate_qt_quick_ui)
        self.generate_qt_quick_ui_layout.addWidget(self.btn_generate_qt_quick_ui)

        self.second_row_button_layout.addWidget(self.generate_qt_quick_ui_group_box)

        # disable preview and generate buttons if list button is not clicked once

        self.btn_preview_qt_quick_ui.setEnabled(False)
        self.btn_generate_qt_quick_ui.setEnabled(False)

        def enable_qt_quick_ui_buttons():
            self.btn_preview_qt_quick_ui.setEnabled(True)
            self.btn_generate_qt_quick_ui.setEnabled(True)

        self.btn_list_qt_quick_ui.clicked.connect(enable_qt_quick_ui_buttons)

        # Generate kf6 kirigami ui

        self.generate_kf6_kirigami_ui_group_box = QGroupBox()
        self.generate_kf6_kirigami_ui_group_box.setTitle("Generate KF6 Kirigami UI")
        self.generate_kf6_kirigami_ui_layout = QVBoxLayout()
        self.generate_kf6_kirigami_ui_group_box.setLayout(self.generate_kf6_kirigami_ui_layout)

        self.btn_list_kf6_kirigami_ui = QPushButton("List", self)
        self.btn_list_kf6_kirigami_ui.clicked.connect(self.list_kf6_kirigami_ui)
        self.generate_kf6_kirigami_ui_layout.addWidget(self.btn_list_kf6_kirigami_ui)

        self.btn_preview_kf6_kirigami_ui = QPushButton("Preview", self)
        self.btn_preview_kf6_kirigami_ui.clicked.connect(self.preview_kf6_kirigami_ui)
        self.generate_kf6_kirigami_ui_layout.addWidget(self.btn_preview_kf6_kirigami_ui)

        self.btn_generate_kf6_kirigami_ui = QPushButton("Generate", self)
        self.btn_generate_kf6_kirigami_ui.clicked.connect(self.generate_kf6_kirigami_ui)
        self.generate_kf6_kirigami_ui_layout.addWidget(self.btn_generate_kf6_kirigami_ui)

        self.second_row_button_layout.addWidget(self.generate_kf6_kirigami_ui_group_box)

        # disable preview and generate buttons if list button is not clicked once

        self.btn_preview_kf6_kirigami_ui.setEnabled(False)
        self.btn_generate_kf6_kirigami_ui.setEnabled(False)

        def enable_kf6_kirigami_ui_buttons():
            self.btn_preview_kf6_kirigami_ui.setEnabled(True)
            self.btn_generate_kf6_kirigami_ui.setEnabled(True)

        self.btn_list_kf6_kirigami_ui.clicked.connect(enable_kf6_kirigami_ui_buttons)

        # Generate "QML imports integration"

        self.generate_qml_group_box = QGroupBox()
        self.generate_qml_group_box.setTitle("Generate QML Imports Integration")
        self.generate_qml_layout = QVBoxLayout()
        self.generate_qml_group_box.setLayout(self.generate_qml_layout)

        self.btn_list_qml = QPushButton("List", self)
        self.btn_list_qml.clicked.connect(self.list_qml_imports)
        self.generate_qml_layout.addWidget(self.btn_list_qml)

        self.btn_preview_qml = QPushButton("Preview", self)
        self.btn_preview_qml.clicked.connect(self.preview_qml_imports)
        self.generate_qml_layout.addWidget(self.btn_preview_qml)

        self.btn_generate_qml = QPushButton("Generate", self)
        self.btn_generate_qml.clicked.connect(self.generate_qml_imports)
        self.generate_qml_layout.addWidget(self.btn_generate_qml)

        self.second_row_button_layout.addWidget(self.generate_qml_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_qml.setEnabled(False)
        self.btn_generate_qml.setEnabled(False)

        def enable_qml_buttons():
            self.btn_preview_qml.setEnabled(True)
            self.btn_generate_qml.setEnabled(True)

        self.btn_list_qml.clicked.connect(enable_qml_buttons)


        # generate all

        self.generate_all_group_box = QGroupBox()
        self.generate_all_group_box.setTitle("Generate All")
        self.generate_all_layout = QVBoxLayout()
        self.generate_all_group_box.setLayout(self.generate_all_layout)

        self.btn_list_all = QPushButton("List All", self)
        self.btn_list_all.clicked.connect(self.list_all)
        self.generate_all_layout.addWidget(self.btn_list_all)

        self.btn_preview_all = QPushButton("Preview All", self)
        self.btn_preview_all.clicked.connect(self.preview_all)
        self.generate_all_layout.addWidget(self.btn_preview_all)

        self.btn_generate_all = QPushButton("Generate All", self)
        self.btn_generate_all.clicked.connect(self.generate_all)
        self.generate_all_layout.addWidget(self.btn_generate_all)

        self.second_row_button_layout.addWidget(self.generate_all_group_box)

        # disable preview and generate buttons if list button is not clicked once
        self.btn_preview_all.setEnabled(False)
        self.btn_generate_all.setEnabled(False)

        def enable_all_buttons():
            self.btn_preview_all.setEnabled(True)
            self.btn_generate_all.setEnabled(True)

        self.btn_list_all.clicked.connect(enable_all_buttons)

        # add to layout

        button_widget = QWidget()
        button_layout = QVBoxLayout()
        button_layout.addLayout(self.first_row_button_layout)
        button_layout.addLayout(self.second_row_button_layout)
        button_widget.setLayout(button_layout)
        self.central_layout.addWidget(button_widget)
        self.central_layout.setStretch(0, 1)

        # select qleany file
        if self.manifest_file == "":
            self.select_qleany_manifest_file()
        elif not Path(self.manifest_file).exists():
            self.select_qleany_manifest_file()
        # set root path str
        self.root_path = str(Path(self.manifest_file).parent.resolve())

        # create temp manifest file under temp folder
        self.last_manifest_mtime = os.path.getmtime(self.manifest_file)

        self.create_temp_manifest_file()

        self.load_data()
        self.load_settings()
        self.refresh()
        self.timer.start()

    def on_manifest_file_changed(self, path):
        if self.manifest_refreshed_displayed:
            self.refresh()
            return
        self.manifest_refreshed_displayed = True
        QMessageBox.information(
            None, "File Changed", f"The file {path} has been changed. Refreshing now..."
        )
        self.manifest_refreshed_displayed = False

    def check_manifest_file(self):
        new_mtime = os.path.getmtime(self.manifest_file)
        if new_mtime != self.last_manifest_mtime:
            self.on_manifest_file_changed(self.manifest_file)
            self.last_manifest_mtime = new_mtime

    def clear_preview_folder(self):
        preview_path = Path(__file__).resolve().parent / "preview"
        if preview_path.exists():
            shutil.rmtree(preview_path)
        os.mkdir(preview_path)

    def refresh(self):
        self.clear_preview_folder()
        self.create_temp_manifest_file()
        self.load_data()
        self.load_settings()
        if self.manifest_file != "":
            self.settings.setValue("last_selected_manifest_path", self.manifest_file)
            self.settings.sync()
            self.generate_qml_group_box.setEnabled(qml_imports_generator.is_qml_imports_integration_enabled(self.manifest_file))
            self.generate_qt_widgets_ui_group_box.setEnabled(qt_widgets_generator.is_enabled(self.manifest_file))
            self.generate_qt_quick_ui_group_box.setEnabled(qt_quick_generator.is_enabled(self.manifest_file))
            self.generate_kf6_kirigami_ui_group_box.setEnabled(kf6_kirigami_generator.is_enabled(self.manifest_file))

    def open_entity_relationship_window(self):
        self.relationship_viewer_window = (
            entity_relationship_viewer.EntityRelationshipWindow(self.manifest_file)
        )
        self.relationship_viewer_window.show()

    # all

    def list_all(self):
        list = []
        list.extend(
            root_generator.get_files_to_be_generated(self.temp_manifest_file)
        )
        list.extend(
            entities_generator.get_files_to_be_generated(self.temp_manifest_file)
        )
        list.extend(dto_generator.get_files_to_be_generated(self.temp_manifest_file))
        list.extend(
            repositories_generator.get_files_to_be_generated(self.temp_manifest_file)
        )
        list.extend(cqrs_generator.get_files_to_be_generated(self.temp_manifest_file))
        list.extend(
            presenter_generator.get_files_to_be_generated(self.temp_manifest_file)
        )
        list.extend(
            controller_generator.get_files_to_be_generated(self.temp_manifest_file)
        )
        list.extend(
            application_generator.get_files_to_be_generated(self.temp_manifest_file)
        )
        if qml_imports_generator.is_qml_imports_integration_enabled(self.temp_manifest_file):
            folder_path = qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file)
            list.extend(qml_imports_generator.get_files_to_be_generated(self.temp_manifest_file, {}, folder_path))

        if qt_widgets_generator.is_enabled(self.manifest_file):
            list.extend(
                qt_widgets_generator.get_files_to_be_generated(self.temp_manifest_file)
            )

        if qt_quick_generator.is_enabled(self.manifest_file):
            list.extend(
                qt_quick_generator.get_files_to_be_generated(self.manifest_file)
            )
        if kf6_kirigami_generator.is_enabled(self.manifest_file):
            list.extend(
                kf6_kirigami_generator.get_files_to_be_generated(self.manifest_file)
            )

        self.text_box.clear()
        self.text_box.setPlainText("All files:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_all(self):
        self.clear_preview_folder()
        self.list_all()
        root_generator.preview_root_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        entities_generator.preview_entity_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        dto_generator.preview_dto_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        repositories_generator.preview_repository_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        cqrs_generator.preview_cqrs_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        presenter_generator.preview_presenter_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        controller_generator.preview_controller_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        application_generator.preview_application_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        if qml_imports_generator.is_qml_imports_integration_enabled(self.temp_manifest_file):
            folder_path = qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file)
            qml_imports_generator.preview_qml_imports_files(
                self.root_path,
                folder_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )

        if qt_widgets_generator.is_enabled(self.manifest_file):
            qt_widgets_generator.preview_qt_widgets_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )

        if qt_quick_generator.is_enabled(self.manifest_file):
            qt_quick_generator.preview_qt_quick_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )

        if kf6_kirigami_generator.is_enabled(self.manifest_file):
            kf6_kirigami_generator.preview_kf6_kirigami_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )

        self.text_box.setPlainText(
            f"Preview folder cleared beforehand. All files previewed at {Path(self.root_path,).resolve()}/qleany_preview/ folder"
        )

    def generate_all(self):
        file_list = []
        file_list.extend(
            root_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        file_list.extend(
            entities_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        file_list.extend(
            dto_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        file_list.extend(
            repositories_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        file_list.extend(
            cqrs_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        file_list.extend(
            controller_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        file_list.extend(
            presenter_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        file_list.extend(
            application_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        )
        if qml_imports_generator.is_qml_imports_integration_enabled(self.temp_manifest_file):
            folder_path = qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file)
            file_list.extend(
                qml_imports_generator.get_files_to_be_generated(
                    self.temp_manifest_file, self.file_list_view.fetch_file_states(), folder_path
                )
            )

        if qt_widgets_generator.is_enabled(self.manifest_file):
            file_list.extend(
                qt_widgets_generator.get_files_to_be_generated(
                    self.temp_manifest_file, self.file_list_view.fetch_file_states()
                )
            )

        if qt_quick_generator.is_enabled(self.manifest_file):
            file_list.extend(
                qt_quick_generator.get_files_to_be_generated(
                    self.temp_manifest_file, self.file_list_view.fetch_file_states()
                )
            )

        if kf6_kirigami_generator.is_enabled(self.manifest_file):
            file_list.extend(
                kf6_kirigami_generator.get_files_to_be_generated(
                    self.temp_manifest_file, self.file_list_view.fetch_file_states()
                )
            )

        if self.display_overwrite_confirmation(file_list):
            # display progress dialog
            progress = QProgressDialog(self)
            progress.setLabelText("Generating files...")
            progress.setRange(0, 12)
            progress.show()
            QCoreApplication.processEvents()


            self.list_all()

            root_generator.generate_root_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(1)            
            QCoreApplication.processEvents()

            entities_generator.generate_entity_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(2)
            QCoreApplication.processEvents()

            dto_generator.generate_dto_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(3)
            QCoreApplication.processEvents()

            repositories_generator.generate_repository_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(4)
            QCoreApplication.processEvents()

            cqrs_generator.generate_cqrs_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(5)
            QCoreApplication.processEvents()

            controller_generator.generate_controller_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(6)
            QCoreApplication.processEvents()

            presenter_generator.generate_presenter_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(7)
            QCoreApplication.processEvents()

            application_generator.generate_application_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            progress.setValue(8)
            QCoreApplication.processEvents()

            if qml_imports_generator.is_qml_imports_integration_enabled(self.temp_manifest_file):
                qml_imports_generator.generate_qml_imports_files(
                    self.root_path,
                    qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file),
                    self.temp_manifest_file,
                    self.file_list_view.fetch_file_states(),
                    self.uncrustify_config_file,
                )
            progress.setValue(9)
            QCoreApplication.processEvents()

            if qt_widgets_generator.is_enabled(self.temp_manifest_file):
                qt_widgets_generator.generate_qt_widgets_files(
                    self.root_path,
                    self.temp_manifest_file,
                    self.file_list_view.fetch_file_states(),
                    self.uncrustify_config_file,
                )
            progress.setValue(10)
            QCoreApplication.processEvents()

            if qt_quick_generator.is_enabled(self.temp_manifest_file):
                qt_quick_generator.generate_qt_quick_files(
                    self.root_path,
                    self.temp_manifest_file,
                    self.file_list_view.fetch_file_states(),
                    self.uncrustify_config_file,
                )
            progress.setValue(11)
            QCoreApplication.processEvents()

            if kf6_kirigami_generator.is_enabled(self.temp_manifest_file):
                kf6_kirigami_generator.generate_kf6_kirigami_files(
                    self.root_path,
                    self.temp_manifest_file,
                    self.file_list_view.fetch_file_states(),
                    self.uncrustify_config_file,
                )
            progress.setValue(12)
            QCoreApplication.processEvents()

            self.text_box.setPlainText("All files generated")

    # root functions
    
    def list_root(self):
        list = root_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Root files:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_root(self):
        self.clear_preview_folder()
        self.list_root()
        root_generator.preview_root_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.setPlainText(
            f"Preview folder cleared beforehand. Root files previewed at {Path(self.root_path,).resolve()}/qleany_preview/ folder"
        )

    def generate_root(self):
        self.list_root()
        if self.display_overwrite_confirmation(
            root_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            root_generator.generate_root_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.setPlainText("Root files generated")

    # entities functions

    def list_entities(self):
        list = entities_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Entities:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_entities(self):
        self.list_entities()
        entities_generator.preview_entity_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" Entities previewed at {Path(__file__).resolve().parent}/preview/ folder"
        )

    def generate_entities(self):
        self.list_entities()
        if self.display_overwrite_confirmation(
            entities_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            entities_generator.generate_entity_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("Entities generated")

    # DTOs functions

    def list_dtos(self):
        list = dto_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("DTOs:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_dtos(self):
        self.list_dtos()
        dto_generator.preview_dto_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" DTOs previewed at {Path(__file__).resolve().parent}/preview/ folder"
        )

    def generate_dtos(self):
        self.list_dtos()
        if self.display_overwrite_confirmation(
            dto_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            dto_generator.generate_dto_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("DTOs generated")

    # Repositories functions

    def list_repositories(self):
        list = repositories_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Repositories:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_repositories(self):
        self.list_repositories()
        repositories_generator.preview_repository_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" Repositories previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_repositories(self):
        self.list_repositories()
        if self.display_overwrite_confirmation(
            repositories_generator.get_files_to_be_generated(
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
            )
        ):
            repositories_generator.generate_repository_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("Repositories generated")

    # CQRS functions

    def list_cqrs(self):
        list = cqrs_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.setPlainText("CQRS:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_cqrs(self):
        self.list_cqrs()
        cqrs_generator.preview_cqrs_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" CQRS files previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_cqrs(self):
        self.list_cqrs()
        if self.display_overwrite_confirmation(
            cqrs_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            cqrs_generator.generate_cqrs_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("CQRS generated")

    # Presenters functions

    def list_presenters(self):
        list = presenter_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Presenters:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_presenters(self):
        self.list_presenters()
        presenter_generator.preview_presenter_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" Presenters previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_presenters(self):
        self.list_presenters()
        if self.display_overwrite_confirmation(
            presenter_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            presenter_generator.generate_presenter_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("Presenters generated")

    # Controllers functions

    def list_controllers(self):
        list = controller_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Controllers to be generated:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_controllers(self):
        self.list_controllers()
        controller_generator.preview_controller_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" Controllers previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_controllers(self):
        self.list_controllers()
        if self.display_overwrite_confirmation(
            controller_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            controller_generator.generate_controller_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("Controllers generated")

    # Application functions

    def list_application(self):
        list = application_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Application:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_application(self):
        self.list_application()
        application_generator.preview_application_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" Application previewed at {Path(__file__).resolve().parent}/preview/ folder"
        )

    def generate_application(self):
        self.list_application()
        if self.display_overwrite_confirmation(
            application_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            application_generator.generate_application_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("Application generated")

    # "QML imports integration" functions

    def list_qml_imports(self):
        folder_path = qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file)
        list = qml_imports_generator.get_files_to_be_generated(self.temp_manifest_file, {}, folder_path)
        self.text_box.clear()
        self.text_box.setPlainText("QML imports integration:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_qml_imports(self):
        self.list_qml_imports()
        folder_path = qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file)
        qml_imports_generator.preview_qml_imports_files(
            self.root_path,
            folder_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" QML files previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_qml_imports(self):
        self.list_qml_imports()
        folder_path = qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file)
        if self.display_overwrite_confirmation(
            qml_imports_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states(), folder_path
            )
        ):
            qml_imports_generator.generate_qml_imports_files(
                self.root_path,
                qml_imports_generator.get_qml_imports_integration_folder_path(self.temp_manifest_file),
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("QML generated")

    # Qt Widgets UI functions
            
    def list_qt_widgets_ui(self):
        list = qt_widgets_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Qt Widgets UI:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_qt_widgets_ui(self):
        self.list_qt_widgets_ui()
        qt_widgets_generator.preview_qt_widgets_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" Qt Widgets UI files previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_qt_widgets_ui(self):
        self.list_qt_widgets_ui()
        if self.display_overwrite_confirmation(
            qt_widgets_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            qt_widgets_generator.generate_qt_widgets_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("Qt Widgets UI generated")

    # Qt Quick UI functions
            
    def list_qt_quick_ui(self):
        list = qt_quick_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("Qt Quick UI:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)

    def preview_qt_quick_ui(self):
        self.list_qt_quick_ui()
        qt_quick_generator.preview_qt_quick_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" Qt Quick UI files previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_qt_quick_ui(self):
        self.list_qt_quick_ui()
        if self.display_overwrite_confirmation(
            qt_quick_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            qt_quick_generator.generate_qt_quick_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("Qt Quick UI generated")

    # KF6 Kirigami UI functions
            
    def list_kf6_kirigami_ui(self):
        list = kf6_kirigami_generator.get_files_to_be_generated(self.temp_manifest_file)
        self.text_box.clear()
        self.text_box.setPlainText("KF6 Kirigami UI:\n\n")
        self.text_box.appendPlainText("\n".join(list))
        self.file_list_view.list_files(list)
        
    def preview_kf6_kirigami_ui(self):
        self.list_kf6_kirigami_ui()
        kf6_kirigami_generator.preview_kf6_kirigami_files(
            self.root_path,
            self.temp_manifest_file,
            self.file_list_view.fetch_file_states(),
            self.uncrustify_config_file,
        )
        self.text_box.clear()
        self.text_box.setPlainText(
            f'Preview folder NOT cleared beforehand. Do it if needed by clicking on "Clear Preview Folder" button.'
        )
        self.text_box.appendPlainText(
            f" KF6 Kirigami UI files previewed at {Path(__file__).resolve().parent}/qleany_preview/ folder"
        )

    def generate_kf6_kirigami_ui(self):
        self.list_kf6_kirigami_ui()
        if self.display_overwrite_confirmation(
            kf6_kirigami_generator.get_files_to_be_generated(
                self.temp_manifest_file, self.file_list_view.fetch_file_states()
            )
        ):
            kf6_kirigami_generator.generate_kf6_kirigami_files(
                self.root_path,
                self.temp_manifest_file,
                self.file_list_view.fetch_file_states(),
                self.uncrustify_config_file,
            )
            self.text_box.clear()
            self.text_box.setPlainText("KF6 Kirigami UI generated")

    def display_overwrite_confirmation(self, files: list):
        # join self.root_path and file
        files = [os.path.join(self.root_path, file) for file in files]

        existing_files = [file for file in files if os.path.isfile(file)]

        # format the file list as a string for display
        fileList = "\n".join(existing_files)

        dialog = QDialog()
        layout = QVBoxLayout(dialog)
        if existing_files:  # if the list has something in it
            label = QLabel()
            label.setText(
                f"The following files exist and will be overwritten:\nAre you sure you want to continue?"
            )

            layout.addWidget(label)

            dialog.setWindowTitle("Overwrite Confirmation")

            # add the plain text edit to the dialog
            textEdit = QPlainTextEdit()
            textEdit.setReadOnly(True)
            textEdit.setPlainText(f"{fileList}\n")
            layout.addWidget(textEdit)
            dialog.resize(700, 300)
        else:
            label = QLabel()
            label.setText(f"Are you sure you want to continue?\n")

            layout.addWidget(label)

            label.setWindowTitle("Confirmation")

        # create Ok and Cancel buttons
        buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        buttonBox.accepted.connect(dialog.accept)
        buttonBox.rejected.connect(dialog.reject)
        layout.addWidget(buttonBox)

        returnValue = dialog.exec()
        if returnValue == QDialog.Accepted:
            return True
        else:
            return False

    # Expand all

    def expand_all(self):
        self.tree.expandAll()

    def create_temp_manifest_file(self):
        with open(self.manifest_file) as file:
            data = yaml.load(file, Loader=yaml.FullLoader)

        # create temp folder if not exists
        Path(self.temp_manifest_file).parent.mkdir(parents=True, exist_ok=True)

        with open(self.temp_manifest_file, "w") as file:
            yaml.dump(data, file)

    def load_data(self):
        with open(self.temp_manifest_file) as file:
            self.data = yaml.load(file, Loader=yaml.FullLoader)

        self.model = self.create_model(self.data)
        self.tree.setModel(self.model)
        self.tree.model().itemChanged.connect(self.handleItemChanged)

        self.tree.setColumnWidth(0, 200)

    def create_model(self, data, parent=None):
        if parent is None:
            parent = QStandardItemModel()
            parent.setHorizontalHeaderLabels(["Key", "Value"])
        if isinstance(data, dict):
            for key, value in data.items():
                if isinstance(value, (dict, list)):
                    item = QStandardItem(key)
                    self.create_model(value, item)
                    parent.appendRow([item, QStandardItem()])
                else:
                    key_item = QStandardItem(str(key))
                    value_item = QStandardItem(str(value))
                    parent.appendRow([key_item, value_item])
        elif isinstance(data, list):
            for i, value in enumerate(data):
                if isinstance(value, (dict, list)):
                    item = QStandardItem(str(i))
                    self.create_model(value, item)
                    parent.appendRow([item, QStandardItem()])
                else:
                    key_item = QStandardItem(str(i))
                    value_item = QStandardItem(str(value))
                    parent.appendRow([key_item, value_item])

        return parent

    def on_tree_item_click(self, index):
        # TODO: Generate the list of files based on item clicked and display in text_box
        # self.text_box.setPlainText("Generated files:")
        pass

    def on_check_item_changed(self, state, key):
        # Load the current state of the YAML file
        with open(self.temp_manifest_file) as file:
            data = yaml.load(file, Loader=yaml.FullLoader)

        # Write the updated state back to the YAML file
        with open(self.temp_manifest_file, "w") as file:
            yaml.dump(data, file)

    def handleItemChanged(self, item):
        # This function will be called whenever a checkbox's state is changed
        if item.isCheckable():
            # We navigate up the tree to construct the keys
            keys = []
            index = item.index()
            while index.isValid():
                text = self.model.itemFromIndex(index).text()
                if not text:
                    text = self.model.itemFromIndex(index).data()
                keys.append(text)
                index = index.parent()
            keys = keys[
                ::-1
            ]  # reverse the keys and exclude the first one, which is the root node

            # Now that we have the keys, let's update the data and the yaml file
            data = self.data
            for key in keys[:-1]:
                if isinstance(data, list):
                    key = int(key)
                data = data[key]
            # The last key corresponds to the checkbox
            data[keys[-1]] = item.checkState() == Qt.Checked

            # Now we update the yaml file
            with open(self.temp_manifest_file, "w") as file:
                yaml.dump(self.data, file)

            self.save_settings()

    def get_generate_items(self, data, path=None):
        if path is None:
            path = []

        if isinstance(data, dict):
            for key, value in data.items():
                new_path = path + [key]
                yield from self.get_generate_items(value, new_path)
        elif isinstance(data, list):
            for index, value in enumerate(data):
                new_path = path + [index]
                yield from self.get_generate_items(value, new_path)

    def save_settings(self):
        # Extracts the "generate" items and their paths
        generate_items = list(self.get_generate_items(self.data))

        # clear the settings file
        with open(self.settings_file, "w") as file:
            file.write("")

        # create temp folder if not exists
        Path(self.settings_file).parent.mkdir(parents=True, exist_ok=True)

        # Saves the items to the settings file
        with open(self.settings_file, "w") as file:
            yaml.dump(generate_items, file)

    def load_settings(self):
        # Load the saved state from the settings file
        try:
            with open(self.settings_file, "r") as file:
                saved_generate_items = yaml.load(file, Loader=yaml.FullLoader)
        except FileNotFoundError:
            return

        # Apply the saved state to self.data

        for path, value in saved_generate_items:
            for key in path[:-1]:
                try:
                    item = self.data[key]
                except KeyError:
                    break
                except IndexError:
                    break

            self.data[path[-1]] = value

        # Update self.temp_manifest_file
        with open(self.temp_manifest_file, "w") as file:
            yaml.dump(self.data, file)

        # Update the tree view
        self.model = self.create_model(self.data)
        self.tree.setModel(self.model)
        self.tree.model().itemChanged.connect(self.handleItemChanged)

    def show_launch_dialog(self):
        title = "Welcome to Qleany, the Qt Clean Architecture generator GUI!"

        message = """
        Cute Clean Architecture generator GUI\n
        \n
        This little application is a GUI for the generator. It allows you to select which file to generate in the middle view.\n
        It also allows you to preview the files in the "preview" folder by the python script before generating them properly.\n
        This GUI uses the qleany.yaml file placed at the root of your project and you can cherry-pick which files to generate.\n
        The states of the checkboxes are saved in the settings.yaml file.\n
        The qleany.yaml file is not modified by this UI.\n
        The manifest_temp.yaml file is a modified copy of the qleany.yaml file, but exists only for argument passing to the generator scripts.\n
        Any modification of the qleany.yaml file will be reflected in the UI.\n
        """
        settings = QSettings()
        checkbox = QCheckBox("Do not show this message again.")
        checkbox.setChecked(not settings.value("show_dialog", True, bool))
        msgBox = QMessageBox(self)
        msgBox.setText(message)
        msgBox.setWindowTitle(title)
        msgBox.setCheckBox(checkbox)

        # Show the message box at the foreground of the main window
        msgBox.setWindowModality(Qt.WindowModal)

        msgBox.exec()

        # Save the user's preference
        settings.setValue("show_dialog", not checkbox.isChecked())

    def open_menu(self, position):
        indexes = self.tree.selectedIndexes()
        if indexes:
            level = 0
            index = indexes[0]
            while index.parent().isValid():
                index = index.parent()
                level += 1

            menu = QMenu()
            if level == 0:
                action = menu.addAction("Expand All")
                action.triggered.connect(self.expand_completely_item)
                # action = menu.addAction("Collapse All")
                # action.triggered.connect(self.collapse_completely_item)
            menu.exec_(self.tree.viewport().mapToGlobal(position))

    def expand_completely_item(self):
        selected = self.tree.selectionModel().selectedIndexes()[0]
        self.tree.expandRecursively(selected)

    # def collapse_completely_item(self):
    #     selected = self.tree.selectionModel().selectedIndexes()[0]
    #     self.tree.collapseRecursively(selected)

    def select_qleany_manifest_file(self):
        start_path = self.settings.value("last_selected_manifest_path", "")
        options = QFileDialog.Options()
        options |= QFileDialog.ReadOnly

        file_name, _ = QFileDialog.getOpenFileName(
            self,
            "Select qleany.yaml",
            start_path,
            "YAML Files (*.yml, *.yaml);;All Files (*)",
            options=options,
        )

        if file_name:
            self.settings.setValue("last_selected_manifest_path", file_name)
            self.manifest_file_text.setText(file_name)
            self.manifest_file = file_name
            self.root_path = str(Path(self.manifest_file).parent.resolve())
            self.refresh()
            self.list_all()


def main():
    full_path = Path(__file__).resolve().parent

    # add the current directory to the path so that we can import the generated files
    sys.path.append(full_path)

    # set the current directory to the generator directory
    os.chdir(full_path)

    app = QApplication(sys.argv)

    QCoreApplication.setOrganizationName("qleany-generator")
    QCoreApplication.setOrganizationDomain("qleany-generator.eu")
    QCoreApplication.setApplicationName("qleany-generator")

    window = MainWindow()
    # make the window stay on top
    window.show()

    # Load settings and show the launch dialog
    settings = QSettings()
    show_dialog = settings.value("show_dialog", True, bool)

    if show_dialog:
        window.show_launch_dialog()

    sys.exit(app.exec())


# List view with checkboxes


class CheckableFileListView(QWidget):
    def __init__(self, parent=None):
        super(CheckableFileListView, self).__init__(parent)
        self.settings = QSettings()

        self.fileListView = QListView(self)
        self.model = QStandardItemModel(self.fileListView)

        self.checkAllButton = QPushButton("Check All", self)
        self.checkAllButton.clicked.connect(self.check_all)

        self.uncheckAllButton = QPushButton("Uncheck All", self)
        self.uncheckAllButton.clicked.connect(self.uncheck_all)

        layout = QVBoxLayout(self)
        layout.addWidget(self.fileListView)
        layout.addWidget(self.checkAllButton)
        layout.addWidget(self.uncheckAllButton)

        self.fileListView.setModel(self.model)

    def list_files(self, file_paths):
        self.model.clear()

        for file_path in file_paths:
            item = QStandardItem(file_path)
            item.setCheckable(True)

            # Load saved check state, default to checked

            check_state_bool = self.settings.value(
                f"file_check_state/{file_path}", Qt.Checked, type=bool
            )
            if check_state_bool:
                check_state = Qt.Checked
            else:
                check_state = Qt.Unchecked

            item.setCheckState(check_state)

            self.model.appendRow(item)

        # Connect to the itemChanged signal
        self.model.itemChanged.connect(self.handle_item_changed)

    def handle_item_changed(self, item):
        # Save the check state of the item
        self.settings.setValue(
            f"file_check_state/{item.text()}", item.checkState() == Qt.Checked
        )

    def fetch_file_states(self):
        file_states = {}

        for row in range(self.model.rowCount()):
            item = self.model.item(row)
            file_states[item.text()] = item.checkState() == Qt.Checked

        return file_states

    def get_selected_files(self):
        selected_files = []
        for row in range(self.model.rowCount()):
            item = self.model.item(row)
            if item.checkState() == Qt.Checked:
                selected_files.append(item.text())
        return selected_files

    def check_all(self):
        for row in range(self.model.rowCount()):
            item = self.model.item(row)
            item.setCheckState(Qt.Checked)

    def uncheck_all(self):
        for row in range(self.model.rowCount()):
            item = self.model.item(row)
            item.setCheckState(Qt.Unchecked)


# This is the entry point of the script

if __name__ == "__main__":
    full_path = Path(__file__).resolve().parent

    # add the current directory to the path so that we can import the generated files
    sys.path.append(full_path)

    # set the current directory to the generator directory
    os.chdir(full_path)

    main()



from jinja2 import Environment, FileSystemLoader
import yaml
import os
import sys
import stringcase
import shutil
import uncrustify
import clang_format_runner as clang_format_runner
import generation_dict_tools as tools
from pathlib import Path


def _get_generation_dict(manifest_data) -> dict:
    generation_dict = {}

    cmakelists_dict = {}

    application_name = manifest_data.get("global", {}).get(
        "application_name", "example"
    )

    cmakelists_dict["application_name"] = application_name
    cmakelists_dict["application_name_pascalcase"] = stringcase.pascalcase(
        application_name
    )
    cmakelists_dict["application_name_uppercase"] = stringcase.uppercase(
        application_name
    )
    cmakelists_dict["application_name_lowercase"] = stringcase.lowercase(
        application_name
    ).strip("-_ ")

    # get the paths from the manifest file
    cmakelists_dict["entities_path"] = manifest_data.get("entities", {}).get(
        "folder_path", "src/entities"
    )
    cmakelists_dict["contracts_path"] = manifest_data.get("contracts", {}).get(
        "folder_path", "src/contracts"
    )
    cmakelists_dict["persistence_path"] = manifest_data.get("repositories", {}).get(
        "base_folder_path", "src/persistence"
    )
    cmakelists_dict["contracts_dto_path"] = manifest_data.get("DTOs", {}).get(
        "common_cmake_folder_path", "src/contracts.dto"
    )
    cmakelists_dict["contracts_cqrs_path"] = manifest_data.get("CQRS", {}).get(
        "common_cmake_folder_path", "src/contracts.cqrs"
    )
    cmakelists_dict["application_path"] = manifest_data.get("application", {}).get(
        "common_cmake_folder_path", "src/application"
    )
    cmakelists_dict["controller_path"] = manifest_data.get("controller", {}).get(
        "folder_path", "src/controller"
    )
    cmakelists_dict["presenter_path"] = manifest_data.get("presenter", {}).get(
        "folder_path", "src/presenters"
    )

    # get the front ends
    front_ends = manifest_data.get("front_ends", {})
    front_end_dict = {}
    front_end_count = 0

    front_end_dict["qt_widgets"] = front_ends.get("qt_widgets", {})
    front_end_dict["qt_widgets"]["enabled"] = (
        True if front_end_dict["qt_widgets"] else False
    )
    if front_end_dict["qt_widgets"]["enabled"]:
        assert front_end_dict["qt_widgets"][
            "folder_path"
        ], "qt_widgets folder_path is not set in the manifest file"
        front_end_count += 1

    front_end_dict["qt_quick"] = front_ends.get("qt_quick", {})
    front_end_dict["qt_quick"]["enabled"] = (
        True if front_end_dict["qt_quick"] else False
    )
    if front_end_dict["qt_quick"]["enabled"]:
        assert front_end_dict["qt_quick"][
            "folder_path"
        ], "qt_quick folder_path is not set in the manifest file"
        front_end_count += 1

    front_end_dict["kf6_kirigami"] = front_ends.get("kf6_kirigami", {})
    front_end_dict["kf6_kirigami"]["enabled"] = (
        True if front_end_dict["kf6_kirigami"] else False
    )
    if front_end_dict["kf6_kirigami"]["enabled"]:
        assert front_end_dict["kf6_kirigami"][
            "folder_path"
        ], "kf6_kirigami folder_path is not set in the manifest file"
        front_end_count += 1

    front_end_dict["kf6_widgets"] = front_ends.get("kf6_widgets", {})
    front_end_dict["kf6_widgets"]["enabled"] = (
        True if front_end_dict["kf6_widgets"] else False
    )
    if front_end_dict["kf6_widgets"]["enabled"]:
        assert front_end_dict["kf6_widgets"][
            "folder_path"
        ], "kf6_widgets folder_path is not set in the manifest file"
        front_end_count += 1

    front_end_dict["qml_imports_integration"] = front_ends.get(
        "qml_imports_integration", {}
    )
    front_end_dict["qml_imports_integration"]["enabled"] = (
        True if front_end_dict["qml_imports_integration"] else False
    )
    if front_end_dict["qml_imports_integration"]["enabled"]:
        assert front_end_dict["qml_imports_integration"][
            "folder_path"
        ], "qml_imports_integration folder_path is not set in the manifest file"
        front_end_count += 1

    front_end_dict["multiple_uis"] = True if front_end_count > 1 else False
    front_end_dict["no_ui"] = True if front_end_count == 0 else False

    cmakelists_dict["front_ends"] = front_end_dict

    generation_dict["cmakelists"] = cmakelists_dict

    return generation_dict


def _generate_cmakelists_file(
    root_path: str,
    template_path: str,
    cmakelists_dict: dict,
    files_to_be_generated: dict[str, bool],
):
    cmakelists_file = "CMakeLists.txt"

    # Create the jinja2 environment
    template_env = Environment(
        loader=FileSystemLoader(os.path.join("templates/root", template_path))
    )
    # Load the template
    cmakelists_template = template_env.get_template("CMakeLists.txt.jinja2")

    # generate the real cmakelists file if in the files_to_be_generated dict the value is True
    if not files_to_be_generated.get(cmakelists_file, False):
        return

    output_file = os.path.join(root_path, cmakelists_file)

    # Render the template
    output = cmakelists_template.render(
        application_name=cmakelists_dict["application_name"],
        entities_path=cmakelists_dict["entities_path"],
        contracts_path=cmakelists_dict["contracts_path"],
        persistence_path=cmakelists_dict["persistence_path"],
        contracts_dto_path=cmakelists_dict["contracts_dto_path"],
        contracts_cqrs_path=cmakelists_dict["contracts_cqrs_path"],
        application_path=cmakelists_dict["application_path"],
        controller_path=cmakelists_dict["controller_path"],
        presenter_path=cmakelists_dict["presenter_path"],
        front_ends=cmakelists_dict["front_ends"],
    )

    # Create the directory if it does not exist
    os.makedirs(os.path.dirname(output_file), exist_ok=True)

    # Write the output to the file
    with open(output_file, "w") as fh:
        fh.write(output)

    print(f"Successfully wrote file {output_file}")


def _generate_cmakelists_file_for_multiple_uis(
    root_path: str, cmakelists_dict: dict, files_to_be_generated: dict[str, bool]
):
    cmakelists_file = "CMakeLists.txt"

    # Create the jinja2 environment
    template_env = Environment(loader=FileSystemLoader("templates/root"))
    # Load the template
    cmakelists_template = template_env.get_template("CMakeLists.txt.jinja2")

    # generate the real cmakelists file if in the files_to_be_generated dict the value is True
    if not files_to_be_generated.get(cmakelists_file, False):
        return

    output_file = os.path.join(root_path, cmakelists_file)

    # Render the template
    output = cmakelists_template.render(
        application_name=cmakelists_dict["application_name"],
        application_name_upper=stringcase.uppercase(
            cmakelists_dict["application_name"]
        ),
        application_name_lower=stringcase.lowercase(
            cmakelists_dict["application_name"]
        ),
        entities_path=cmakelists_dict["entities_path"],
        contracts_path=cmakelists_dict["contracts_path"],
        persistence_path=cmakelists_dict["persistence_path"],
        contracts_dto_path=cmakelists_dict["contracts_dto_path"],
        contracts_cqrs_path=cmakelists_dict["contracts_cqrs_path"],
        application_path=cmakelists_dict["application_path"],
        controller_path=cmakelists_dict["controller_path"],
        presenter_path=cmakelists_dict["presenter_path"],
        front_ends=cmakelists_dict["front_ends"],
    )

    # Create the directory if it does not exist
    os.makedirs(os.path.dirname(output_file), exist_ok=True)

    # Write the output to the file
    with open(output_file, "w") as fh:
        fh.write(output)

    print(f"Successfully wrote file {output_file}")

def _generate_cmakelists_file_for_no_ui(
    root_path: str, cmakelists_dict: dict, files_to_be_generated: dict[str, bool]
):
    cmakelists_file = "CMakeLists.txt"

    # Create the jinja2 environment
    template_env = Environment(loader=FileSystemLoader("templates/root"))
    # Load the template
    cmakelists_template = template_env.get_template("CMakeLists.txt.no_ui.jinja2")

    # generate the real cmakelists file if in the files_to_be_generated dict the value is True
    if not files_to_be_generated.get(cmakelists_file, False):
        return

    output_file = os.path.join(root_path, cmakelists_file)

    # Render the template
    output = cmakelists_template.render(
        application_name=cmakelists_dict["application_name"],
        application_name_upper=stringcase.uppercase(
            cmakelists_dict["application_name"]
        ),
        application_name_lower=stringcase.lowercase(
            cmakelists_dict["application_name"]
        ),
        entities_path=cmakelists_dict["entities_path"],
        contracts_path=cmakelists_dict["contracts_path"],
        persistence_path=cmakelists_dict["persistence_path"],
        contracts_dto_path=cmakelists_dict["contracts_dto_path"],
        contracts_cqrs_path=cmakelists_dict["contracts_cqrs_path"],
        application_path=cmakelists_dict["application_path"],
        controller_path=cmakelists_dict["controller_path"],
        presenter_path=cmakelists_dict["presenter_path"],
        front_ends=cmakelists_dict["front_ends"],
    )

    # Create the directory if it does not exist
    os.makedirs(os.path.dirname(output_file), exist_ok=True)

    # Write the output to the file
    with open(output_file, "w") as fh:
        fh.write(output)

    print(f"Successfully wrote file {output_file}")

def _generate_kf6_kirigami_files(
    root_path: str, cmakelists_dict: dict, files_to_be_generated: dict[str, bool]
):
    application_name = cmakelists_dict["application_name"]
    application_name_lower = cmakelists_dict["application_name_lowercase"]
    application_name_pascal = cmakelists_dict["application_name_pascalcase"]

    # Create the jinja2 environment
    template_env = Environment(
        loader=FileSystemLoader(os.path.join("templates/root", "kf6_kirigami"))
    )

    # generate the real files if in the files_to_be_generated dict the value is True
    if files_to_be_generated.get(f"org.kde.{application_name_lower}.json", False):
        # Load the template
        org_kde_json_template = template_env.get_template("org.kde.placeholder.json.jinja2")

        output_file = os.path.join(root_path, f"org.kde.{application_name_lower}.json")

        # Render the template
        output = org_kde_json_template.render(
            application_name_lower=application_name_lower
        )

        # Create the directory if it does not exist
        os.makedirs(os.path.dirname(output_file), exist_ok=True)

        # Write the output to the file
        with open(output_file, "w") as fh:
            fh.write(output)

        print(f"Successfully wrote file {output_file}")

    if files_to_be_generated.get(f"org.kde.{application_name_lower}.desktop", False):
        # Load the template
        desktop_template = template_env.get_template("org.kde.placeholder.desktop.jinja2")

        output_file = os.path.join(
            root_path, f"org.kde.{application_name_lower}.desktop"
        )

        # Render the template
        output = desktop_template.render(
            application_name=cmakelists_dict["application_name"]
        )

        # Create the directory if it does not exist
        os.makedirs(os.path.dirname(output_file), exist_ok=True)

        # Write the output to the file
        with open(output_file, "w") as fh:
            fh.write(output)

        print(f"Successfully wrote file {output_file}")

    if files_to_be_generated.get(f"org.kde.{application_name_lower}.metainfo.xml", False):
        # Load the template
        metainfo_template = template_env.get_template("org.kde.placeholder.metainfo.xml.jinja2")

        output_file = os.path.join(
            root_path, f"org.kde.{application_name_lower}.metainfo.xml"
        )

        # Render the template
        output = metainfo_template.render(
            application_name=cmakelists_dict["application_name"],
            application_name_lower=application_name_lower
        )

        # Create the directory if it does not exist
        os.makedirs(os.path.dirname(output_file), exist_ok=True)

        # Write the output to the file
        with open(output_file, "w") as fh:
            fh.write(output)

        print(f"Successfully wrote file {output_file}")

    if files_to_be_generated.get(f"{application_name_pascal}.kdev4", False):
        # Load the template
        kdev4_template = template_env.get_template("placeholder.kdev4.jinja2")

        output_file = os.path.join(root_path, f"{application_name_pascal}.kdev4")

        # Render the template
        output = kdev4_template.render(application_name_pascal=application_name_pascal)

        # Create the directory if it does not exist
        os.makedirs(os.path.dirname(output_file), exist_ok=True)

        # Write the output to the file
        with open(output_file, "w") as fh:
            fh.write(output)

        print(f"Successfully wrote file {output_file}")

    # Create the jinja2 environment for android files
    template_env = Environment(
        loader=FileSystemLoader(os.path.join("templates/root", "kf6_kirigami", "android"))
    )

    if files_to_be_generated.get(f"android/AndroidManifest.xml", False):
        # Load the template
        android_manifest_template = template_env.get_template("AndroidManifest.xml.jinja2")

        output_file = os.path.join(root_path, "android/AndroidManifest.xml")

        # Render the template
        output = android_manifest_template.render(
            application_name_lower=application_name_lower,
            application_name=application_name
        )

        # Create the directory if it does not exist
        os.makedirs(os.path.dirname(output_file), exist_ok=True)

        # Write the output to the file
        with open(output_file, "w") as fh:
            fh.write(output)

        print(f"Successfully wrote file {output_file}")

    # copy the other android and LICENSES files if they are in the files_to_be_generated dict
    if files_to_be_generated.get(f"android/build.gradle", False):
        os.makedirs(os.path.join(root_path, "android"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/android", "build.gradle"),
            os.path.join(root_path, "android/build.gradle"),
        )
        print(f"Successfully wrote file {root_path}/android/build.gradle")

    if files_to_be_generated.get(f"android/version.gradle.in", False):
        os.makedirs(os.path.join(root_path, "android"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/android", "version.gradle.in"),
            os.path.join(root_path, "android/version.gradle.in"),
        )
        print(f"Successfully wrote file {root_path}/android/version.gradle.in")

    if files_to_be_generated.get(f"android/res/drawable/logo.png", False):
        # Create the directory if it does not exist
        os.makedirs(os.path.join(root_path, "android/res/drawable/"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/android/res/drawable", "logo.png"),
            os.path.join(root_path, "android/res/drawable/logo.png"),
        )
        print(f"Successfully wrote file {root_path}/android/res/drawable/logo.png")

    if files_to_be_generated.get(f"android/res/drawable/splash.xml", False):
        # Create the directory if it does not exist
        os.makedirs(os.path.join(root_path, "android/res/drawable/"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/android/res/drawable", "splash.xml"),
            os.path.join(root_path, "android/res/drawable/splash.xml"),
        )
        print(f"Successfully wrote file {root_path}/android/res/drawable/splash.xml")

    if files_to_be_generated.get(f"LICENSES/BSD-3-Clause.txt", False):
        # Create the directory if it does not exist
        os.makedirs(os.path.join(root_path, "LICENSES"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/LICENSES", "BSD-3-Clause.txt"),
            os.path.join(root_path, "LICENSES/BSD-3-Clause.txt"),
        )
        print(f"Successfully wrote file {root_path}/LICENSES/BSD-3-Clause.txt")

    if files_to_be_generated.get(f"LICENSES/GPL-2.0-or-later.txt", False):
        # Create the directory if it does not exist
        os.makedirs(os.path.join(root_path, "LICENSES"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/LICENSES", "GPL-2.0-or-later.txt"),
            os.path.join(root_path, "LICENSES/GPL-2.0-or-later.txt"),
        )
        print(f"Successfully wrote file {root_path}/LICENSES/GPL-2.0-or-later.txt")

    if files_to_be_generated.get(f"LICENSES/CC0-1.0.txt", False):
        # Create the directory if it does not exist
        os.makedirs(os.path.join(root_path, "LICENSES"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/LICENSES", "CC0-1.0.txt"),
            os.path.join(root_path, "LICENSES/CC0-1.0.txt"),
        )
        print(f"Successfully wrote file {root_path}/LICENSES/CC0-1.0.txt")

    if files_to_be_generated.get(f"LICENSES/FSFAP.txt", False):
        # Create the directory if it does not exist
        os.makedirs(os.path.join(root_path, "LICENSES"), exist_ok=True)
        shutil.copy(
            os.path.join("templates/root/kf6_kirigami/LICENSES", "FSFAP.txt"),
            os.path.join(root_path, "LICENSES/FSFAP.txt"),
        )
        print(f"Successfully wrote file {root_path}/LICENSES/FSFAP.txt")


def generate_root_files(
    root_path: str,
    manifest_file: str,
    files_to_be_generated: dict[str, bool] = {},
    uncrustify_config_file: str = None,
):

    with open(manifest_file, "r") as stream:
        try:
            manifest_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
            return

    generation_dict = _get_generation_dict(manifest_data)

    # generate the CMakeLists.txt file
    if generation_dict["cmakelists"]["front_ends"]["multiple_uis"]:
        _generate_cmakelists_file_for_multiple_uis(
            root_path, generation_dict["cmakelists"], files_to_be_generated
        )
    elif generation_dict["cmakelists"]["front_ends"]["no_ui"]:
        _generate_cmakelists_file_for_no_ui(
            root_path, generation_dict["cmakelists"], files_to_be_generated
        )
    elif generation_dict["cmakelists"]["front_ends"]["qt_widgets"]["enabled"]:
        _generate_cmakelists_file(
            root_path, "qt_widgets", generation_dict["cmakelists"], files_to_be_generated
        )
    elif generation_dict["cmakelists"]["front_ends"]["qt_quick"]["enabled"]:
        _generate_cmakelists_file(
            root_path, "qt_quick", generation_dict["cmakelists"], files_to_be_generated
        )
    # generate the kf6 kirigami files
    elif generation_dict["cmakelists"]["front_ends"]["kf6_kirigami"]["enabled"]:
        _generate_cmakelists_file(
            root_path, "kf6_kirigami", generation_dict["cmakelists"], files_to_be_generated
        )

    if generation_dict["cmakelists"]["front_ends"]["kf6_kirigami"]["enabled"]:
        _generate_kf6_kirigami_files(
            root_path, generation_dict["cmakelists"], files_to_be_generated
        )


def _get_files_to_be_generated_for_qt_widgets(generation_dict: dict) -> list[str]:
    return []


def _get_files_to_be_generated_for_qt_quick(generation_dict: dict) -> list[str]:
    return []


def _get_files_to_be_generated_for_kf6_widgets(generation_dict: dict) -> list[str]:
    application_name = generation_dict["cmakelists"]["application_name"]
    
    # trow an exception
    raise Exception("kf6_widgets is not implemented yet")
    
    return []


def _get_files_to_be_generated_for_kf6_kirigami(generation_dict: dict) -> list[str]:
    application_name_lower = generation_dict["cmakelists"]["application_name_lowercase"]
    application_name_pascal = generation_dict["cmakelists"][
        "application_name_pascalcase"
    ]

    file_list = [
        f"org.kde.{application_name_lower}.json",
        f"{application_name_lower}.desktop",
        f"org.kde.{application_name_lower}.metainfo.xml",
        f"{application_name_pascal}.kdev4",
        f"android/AndroidManifest.xml",
        f"android/build.gradle",
        f"android/version.gradle.in",
        f"android/res/drawable/logo.png",
        f"android/res/drawable/splash.xml",
        f"LICENSES/BSD-3-Clause.txt",
        f"LICENSES/GPL-2.0-or-later.txt",
        f"LICENSES/CC0-1.0.txt",
        f"LICENSES/FSFAP.txt",
    ]
    return file_list


def get_files_to_be_generated(
    manifest_file: str, files_to_be_generated: dict[str, bool] = {}
) -> list[str]:
    """
    Get the list of files that need to be generated based on the manifest file
    """

    with open(manifest_file, "r") as stream:
        try:
            manifest_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
            return

    generation_dict = _get_generation_dict(manifest_data)

    files = []
    # there is always a root CMakeLists.txt file
    files.append("CMakeLists.txt")

    # if multiple UIs:
    if generation_dict["cmakelists"]["front_ends"]["multiple_uis"]:
        if generation_dict["cmakelists"]["front_ends"]["qt_widgets"]["enabled"]:
            files.extend(_get_files_to_be_generated_for_qt_widgets(generation_dict))
        if generation_dict["cmakelists"]["front_ends"]["qt_quick"]["enabled"]:
            files.extend(_get_files_to_be_generated_for_qt_quick(generation_dict))
        if generation_dict["cmakelists"]["front_ends"]["kf6_widgets"]["enabled"]:
            files.extend(_get_files_to_be_generated_for_kf6_widgets(generation_dict))
        if generation_dict["cmakelists"]["front_ends"]["kf6_kirigami"]["enabled"]:
            files.extend(_get_files_to_be_generated_for_kf6_kirigami(generation_dict))

        # remove duplicates
        files = list(dict.fromkeys(files))

    # if only one UI:
    elif generation_dict["cmakelists"]["front_ends"]["qt_widgets"]["enabled"]:
        pass
    elif generation_dict["cmakelists"]["front_ends"]["qt_quick"]["enabled"]:
        pass
    elif generation_dict["cmakelists"]["front_ends"]["kf6_widgets"]["enabled"]:
        files.extend(_get_files_to_be_generated_for_kf6_widgets(generation_dict))
    elif generation_dict["cmakelists"]["front_ends"]["kf6_kirigami"]["enabled"]:
        files.extend(_get_files_to_be_generated_for_kf6_kirigami(generation_dict))

    # strip from files if the value in files_to_be_generated is False
    if files_to_be_generated:
        for path, generate in files_to_be_generated.items():
            if not generate and path in files:
                files.remove(path)

    return files


# generate the files into the preview folder
def preview_root_files(
    root_path: str,
    manifest_file: str,
    files_to_be_generated: dict[str, bool] = {},
    uncrustify_config_file: str = None,
):
    manifest_preview_file = "temp/manifest_preview.yaml"

    # make a copy of the manifest file into temp/manifest_preview.yaml
    shutil.copy(manifest_file, manifest_preview_file)

    # modify the manifest file to generate the files into the preview folder
    with open(manifest_preview_file, "r") as fh:
        manifest = yaml.safe_load(fh)

    # write the modified manifest file
    with open(manifest_preview_file, "w") as fh:
        yaml.dump(manifest, fh)

    root_path = os.path.join(root_path, "qleany_preview")

    # remove .. from the path
    if files_to_be_generated:
        preview_files_to_be_generated = {}
        for path, value in files_to_be_generated.items():
            preview_files_to_be_generated[path.replace("..", "")] = value

        generate_root_files(
            root_path,
            manifest_preview_file,
            preview_files_to_be_generated,
            uncrustify_config_file,
        )

    else:
        generate_root_files(
            root_path, manifest_preview_file, {}, uncrustify_config_file
        )


# Main execution
if __name__ == "__main__":
    full_path = Path(__file__).resolve().parent

    # Add the current directory to the path so that we can import the generated files
    sys.path.append(full_path)

    # Set the current directory to the generator directory
    os.chdir(full_path)

    if len(sys.argv) > 1:
        manifest_arg = sys.argv[1]
        if manifest_arg.endswith(".yaml") or manifest_arg.endswith(".yml"):
            manifest_file = manifest_arg
            root_path = Path(manifest_file).parent

            if len(sys.argv) > 2 and sys.argv[2] == "--preview":
                preview_root_files(root_path, manifest_file)
            else:
                generate_root_files(root_path, manifest_file)
        else:
            print("Error: Manifest file must be named 'qleany.yaml' or 'qleany.yml'")
    else:
        print("Error: Please provide the manifest file as an argument")



import subprocess

import subprocess


def run_clang_format(file_path: str):
    try:
        # Run uncrustify on the file
        result = subprocess.run(
            [
                "clang-format",
                "-i",
                "--style=file",
                file_path,
            ],
            check=True,
        )

        # If the return code is 0, then clang-format ran successfully
        if result.returncode == 0:
            return
            print(f"clang-format ran successfully on {file_path}.")
        else:
            print(f"clang-format failed on {file_path}.")

    except Exception as e:
        print(f"Error running clang-format: {e}")



from jinja2 import Environment, FileSystemLoader
import yaml
import os
import sys
import stringcase
import shutil
import copy
import uncrustify
import clang_format_runner as clang_format_runner
from pathlib import Path
from collections import OrderedDict
import generation_dict_tools as tools


def get_dto_dict_and_feature_ordered_dict(
    feature_by_name: dict, entities_by_name: dict
) -> tuple[dict, OrderedDict]:

    def determine_dto_dependencies_from_fields(fields: list) -> list:
        dto_dependencies = []

        for field in fields:
            if field["is_foreign"]:
                dto_dependencies.append(field["foreign_dto_type"])

        return dto_dependencies

    dto_dict = {}

    for feature in feature_by_name.values():
        # fetch basic entity DTOs

        feature_dto_data = feature.get("DTO", {})
        dto_identical_to_entity = feature_dto_data.get("dto_identical_to_entity", {})

        entity_mappable_with = dto_identical_to_entity.get("entity_mappable_with", "")
        generate_dto_identical_to_entity = dto_identical_to_entity.get(
            "enabled", False
        ) and dto_identical_to_entity.get("entity_mappable_with", "")

        # create DTO entry without foreign DTOs

        if generate_dto_identical_to_entity:
            dto_type_name = f"{stringcase.pascalcase(entity_mappable_with)}DTO"

            dto_dict[dto_type_name] = {
                "feature_name": stringcase.pascalcase(feature["name"]),
                "entity_mappable_with": entity_mappable_with,
                "file_name": f"{stringcase.snakecase(feature['name'])}_dto.h",
                "is_relationship_dto": False,
                "fields": [],
            }

            # add fields without foreign entities
            fields = tools.get_fields_without_foreign_entities(
                entities_by_name[entity_mappable_with]["fields"],
                entities_by_name,
                entity_mappable_with,
            )
            # remove if hidden
            fields = [field for field in fields if not field["hidden"]]

            dto_dict[dto_type_name]["fields"] = fields

            dto_dict[dto_type_name]["dto_dependencies"] = []

        # create DTO entry with foreign DTOs (details)

        if generate_dto_identical_to_entity and tools.does_entity_have_relation_fields(
            entity_mappable_with, entities_by_name
        ):
            dto_type_name = f"{stringcase.pascalcase(feature['name'])}WithDetailsDTO"
            dto_dict[dto_type_name] = {
                "feature_name": stringcase.pascalcase(feature["name"]),
                "entity_mappable_with": entity_mappable_with,
                "file_name": f"{stringcase.snakecase(feature['name'])}_with_details_dto.h",
                "is_relationship_dto": False,
                "fields": [],
            }

            # add fields with foreign entities
            fields = tools.get_fields_with_foreign_entities(
                entities_by_name[entity_mappable_with]["fields"],
                entities_by_name,
                entity_mappable_with,
            )
            # remove if hidden
            fields = [field for field in fields if not field["hidden"]]

            dto_dict[dto_type_name]["fields"] = fields
            dto_dict[dto_type_name][
                "dto_dependencies"
            ] = determine_dto_dependencies_from_fields(
                dto_dict[dto_type_name]["fields"]
            )

        # fetch CRUD DTOs

        crud_data = feature.get("CRUD", {})
        crud_enabled = feature.get("CRUD", {}).get("enabled", {})
        entity_mappable_with = crud_data.get("entity_mappable_with", "")

        if crud_data and crud_enabled and entity_mappable_with:
            generate_create = False
            generate_update = False

            if crud_data["enabled"]:
                generate_create = crud_data.get("create", {}).get("enabled", False)
                generate_update = crud_data.get("update", {}).get("enabled", False)
                generate_insert_relation = tools.does_entity_have_relation_fields(
                    entity_mappable_with, entities_by_name, False
                )
                generate_move = crud_data.get("move_in_relative", {}).get(
                    "enabled", False
                )

            if generate_create:
                dto_type_name = f"Create{stringcase.pascalcase(feature['name'])}DTO"
                dto_dict[dto_type_name] = {
                    "feature_name": stringcase.pascalcase(feature["name"]),
                    "entity_mappable_with": entity_mappable_with,
                    "file_name": f"create_{stringcase.snakecase(feature['name'])}_dto.h",
                    "is_relationship_dto": False,
                    "fields": [],
                }

                # add fields with foreign entities but without id
                fields = tools.get_fields_with_foreign_entities(
                    entities_by_name[entity_mappable_with]["fields"],
                    entities_by_name,
                    entity_mappable_with,
                )
                # remove hidden fields
                fields = [field for field in fields if not field["hidden"]]
                dto_dict[dto_type_name]["fields"] = fields

                # remove id field
                dto_dict[dto_type_name]["fields"] = [
                    field
                    for field in dto_dict[dto_type_name]["fields"]
                    if field["name"] != "id"
                ]

                # add owner id
                owner_dict = tools.determine_owner(
                    entity_mappable_with, entities_by_name
                )
                if owner_dict:
                    owner_name_pascal = stringcase.pascalcase(
                        owner_dict.get("name", "")
                    )
                    owner_name_camel = stringcase.camelcase(owner_dict.get("name", ""))
                    dto_dict[dto_type_name]["fields"].append(
                        {
                            "name": f"{owner_name_camel}Id",
                            "type": "int",
                            "pascal_name": f"{owner_name_pascal}Id",
                            "is_foreign": False,
                        }
                    )

                    # add "position" if it has an owner and the field of owner entity is a QList
                    owner = owner_dict.get("name", "")
                    owner_field = owner_dict.get("field", "")

                    if owner_dict.get("is_list", False) and owner_dict.get(
                        "ordered", False
                    ):
                        dto_dict[dto_type_name]["fields"].append(
                            {
                                "name": "position",
                                "type": "int",
                                "pascal_name": "Position",
                                "is_foreign": False,
                            }
                        )

                dto_dict[dto_type_name][
                    "dto_dependencies"
                ] = determine_dto_dependencies_from_fields(
                    dto_dict[dto_type_name]["fields"]
                )

            if generate_update:
                dto_type_name = f"Update{stringcase.pascalcase(feature['name'])}DTO"
                dto_dict[dto_type_name] = {
                    "feature_name": stringcase.pascalcase(feature["name"]),
                    "entity_mappable_with": entity_mappable_with,
                    "file_name": f"update_{stringcase.snakecase(feature['name'])}_dto.h",
                    "is_relationship_dto": False,
                    "fields": [],
                }

                # add fields with foreign entities
                fields = tools.get_fields_with_foreign_entities(
                    entities_by_name[entity_mappable_with]["fields"],
                    entities_by_name,
                    entity_mappable_with,
                )
                # remove hidden fields
                fields = [field for field in fields if not field["hidden"]]

                dto_dict[dto_type_name]["fields"] = fields

                dto_dict[dto_type_name][
                    "dto_dependencies"
                ] = determine_dto_dependencies_from_fields(
                    dto_dict[dto_type_name]["fields"]
                )

            if generate_insert_relation:
                # DTO in
                dto_type_name = f"{stringcase.pascalcase(feature['name'])}RelationDTO"
                dto_dict[dto_type_name] = {
                    "feature_name": stringcase.pascalcase(feature["name"]),
                    "entity_mappable_with": "",
                    "file_name": f"{stringcase.snakecase(feature['name'])}_relation_dto.h",
                    "is_relation_dto": True,
                    "fields": [],
                    "relation_fields": [],
                }

                # add fields
                dto_dict[dto_type_name]["fields"].append(
                    {
                        "name": "id",
                        "pascal_name": "Id",
                        "is_foreign": False,
                        "type": "int",
                    }
                )
                dto_dict[dto_type_name]["fields"].append(
                    {
                        "name": "relationField",
                        "pascal_name": "RelationshipField",
                        "is_foreign": False,
                        "type": "RelationField",
                    }
                )
                dto_dict[dto_type_name]["fields"].append(
                    {
                        "name": "relatedIds",
                        "pascal_name": "RelatedIds",
                        "is_foreign": False,
                        "type": "QList<int>",
                    }
                )
                dto_dict[dto_type_name]["fields"].append(
                    {
                        "name": "position",
                        "pascal_name": "Position",
                        "is_foreign": False,
                        "type": "int",
                    }
                )       

                # add relationship fields

                dto_dict[dto_type_name][
                    "relation_fields"
                ] = tools.get_only_fields_with_foreign_entities(
                    entities_by_name[entity_mappable_with]["fields"],
                    entities_by_name,
                    entity_mappable_with,
                )

                dto_dict[dto_type_name]["dto_dependencies"] = []

        # fetch command DTOs
        for command in feature.get("commands", []):
            command_dto_data = command.get("dto", {})
            dto_in = command_dto_data.get("in", {})
            if dto_in and dto_in.get("enabled", True):
                dto_type_name = f"{dto_in['type_prefix']}DTO"
                dto_dict[dto_type_name] = {
                    "feature_name": stringcase.pascalcase(feature["name"]),
                    "entity_mappable_with": "",
                    "file_name": f"{stringcase.snakecase(dto_in['type_prefix'])}_dto.h",
                    "is_relationship_dto": False,
                    "fields": [],
                }

                # add fields with foreign entities
                fields = tools.get_fields_with_foreign_entities(
                    dto_in["fields"], entities_by_name
                )
                # remove hidden fields
                fields = [field for field in fields if not field["hidden"]]
                dto_dict[dto_type_name]["fields"] = fields

                dto_dict[dto_type_name][
                    "dto_dependencies"
                ] = determine_dto_dependencies_from_fields(
                    dto_dict[dto_type_name]["fields"]
                )

            dto_out = command_dto_data.get("out", {})
            if dto_out and dto_out.get("enabled", True):
                dto_type_name = f"{dto_out['type_prefix']}DTO"
                dto_dict[dto_type_name] = {
                    "feature_name": stringcase.pascalcase(feature["name"]),
                    "entity_mappable_with": "",
                    "file_name": f"{stringcase.snakecase(dto_out['type_prefix'])}_dto.h",
                    "is_relationship_dto": False,
                    "fields": [],
                }

                # add fields with foreign entities
                fields = tools.get_fields_with_foreign_entities(
                    dto_out["fields"], entities_by_name
                )
                # remove hidden fields
                fields = [field for field in fields if not field["hidden"]]
                dto_dict[dto_type_name]["fields"] = fields
                dto_dict[dto_type_name][
                    "dto_dependencies"
                ] = determine_dto_dependencies_from_fields(
                    dto_dict[dto_type_name]["fields"]
                )

        # fetch query DTOs
        for query in feature.get("queries", []):
            query_dto_data = query.get("dto", {})
            dto_in = query_dto_data.get("in", {})
            if dto_in and dto_in.get("enabled", True):
                dto_type_name = f"{dto_in['type_prefix']}DTO"
                dto_dict[dto_type_name] = {
                    "feature_name": stringcase.pascalcase(feature["name"]),
                    "entity_mappable_with": "",
                    "file_name": f"{stringcase.snakecase(dto_in['type_prefix'])}_dto.h",
                    "is_relationship_dto": False,
                    "fields": [],
                }

                # add fields with foreign entities
                fields = tools.get_fields_with_foreign_entities(
                    dto_in["fields"], entities_by_name
                )
                # remove hidden fields
                fields = [field for field in fields if not field["hidden"]]
                dto_dict[dto_type_name]["fields"] = fields

                dto_dict[dto_type_name][
                    "dto_dependencies"
                ] = determine_dto_dependencies_from_fields(
                    dto_dict[dto_type_name]["fields"]
                )

            dto_out = query_dto_data.get("out", {})
            if dto_out and dto_out.get("enabled", True):
                dto_type_name = f"{dto_out['type_prefix']}DTO"
                dto_dict[dto_type_name] = {
                    "feature_name": stringcase.pascalcase(feature["name"]),
                    "entity_mappable_with": "",
                    "file_name": f"{stringcase.snakecase(dto_out['type_prefix'])}_dto.h",
                    "is_relationship_dto": False,
                    "fields": [],
                }

                # add fields with foreign entities
                fields = tools.get_fields_with_foreign_entities(
                    dto_out["fields"], entities_by_name
                )
                # remove hidden fields
                fields = [field for field in fields if not field["hidden"]]
                dto_dict[dto_type_name]["fields"] = fields
                dto_dict[dto_type_name][
                    "dto_dependencies"
                ] = determine_dto_dependencies_from_fields(
                    dto_dict[dto_type_name]["fields"]
                )

    # create header files for DTOs
    for dto_type_name, dto_data in dto_dict.items():
        header_list = []
        for field in dto_data["fields"]:
            if field["type"] in ["QString", "QDateTime", "QUuid", "QUrl"]:
                header_list.append(f"<{field['type']}>")

        dto_dependencies = dto_data.get("dto_dependencies", [])
        for dto_dependency in dto_dependencies:
            if dto_dependency in dto_dict:
                dto_dependency_file_name = dto_dict[dto_dependency]["file_name"]
                header_list.append(
                    f'"{stringcase.snakecase(dto_dependency.replace("DTO", ""))}/{dto_dependency_file_name}"'
                )

        header_list = list(set(header_list))
        header_list.sort()

        dto_data["header_list"] = header_list

    # add "foreign_feature_name" to foreign fields in DTOs
    for dto_type_name, dto_data in dto_dict.items():
        for field in dto_data["fields"]:
            if not field["is_foreign"]:
                continue
            try:
                foreign_dto_type = field["foreign_dto_type"]
                field["foreign_feature_name"] = dto_dict[foreign_dto_type][
                    "feature_name"
                ]
            except KeyError:
                print(
                    f"ERROR: DTO \"{dto_type_name}\" has a foreign field \"{field['name']}\" with foreign DTO type \"{foreign_dto_type}\" that does not exist. Maybe you forgot to create the corresponding feature ?"
                )
            # dto_data["fields"][i] = field

    # create a new dict "feature_dto_dict" with only the DTOs that are needed for the feature. Each key is a feature name and each value is a dict.
    # The dict contains "dtos" with the list of DTOs. The dict also contains a "feature_dependencies" list with the features that the feature depends on.

    feature_dto_dict = {}
    for dto_type_name, dto_data in dto_dict.items():
        feature_name = dto_data["feature_name"]
        if feature_name not in feature_dto_dict:
            feature_dto_dict[feature_name] = {
                "feature_dependencies": [],
                "dtos": [],
            }

        if not feature_dto_dict[feature_name].get("dtos", {}):
            feature_dto_dict[feature_name]["dtos"] = {}
        feature_dto_dict[feature_name]["dtos"][dto_type_name] = dto_data

        if dto_data["dto_dependencies"]:
            feature_dto_dict[feature_name]["feature_dependencies"].extend(
                dto_data["dto_dependencies"]
            )

    # replace the DTOs in "feature_dependencies" with the feature name of each of these DTOs
    for feature_name, feature_data in feature_dto_dict.items():
        for i, feature_dependency in enumerate(feature_data["feature_dependencies"]):
            feature_data["feature_dependencies"][i] = dto_dict[feature_dependency][
                "feature_name"
            ]

    # remove duplicates from feature_dependencies
    for feature_name, feature_data in feature_dto_dict.items():
        feature_data["feature_dependencies"] = list(
            set(feature_data["feature_dependencies"])
        )

    # order the features by their dependencies, so that the features that depend on other features are generated after the features they depend on

    dto_ordered_dict = OrderedDict()

    for feature_name, feature_data in feature_dto_dict.items():
        # case if a dto depends on another dto in the same feature
        if feature_name in feature_data["feature_dependencies"]:
            feature_data["feature_dependencies"].remove(feature_name)

    original_feature_dto_dict = copy.deepcopy(feature_dto_dict)

    while len(feature_dto_dict) > 0:
        for feature_name, feature_data in feature_dto_dict.items():
            # case if a dto depends on another dto in another feature
            if len(feature_data["feature_dependencies"]) == 0:
                dto_ordered_dict[feature_name] = {}
                dto_ordered_dict[feature_name]["dtos"] = feature_data["dtos"]
                dto_ordered_dict[feature_name][
                    "feature_dependencies"
                ] = original_feature_dto_dict[feature_name]["feature_dependencies"]

                del feature_dto_dict[feature_name]
                for feature_name2, feature_data2 in feature_dto_dict.items():
                    if feature_name in feature_data2["feature_dependencies"]:
                        feature_data2["feature_dependencies"].remove(feature_name)
                break

    return dto_dict, dto_ordered_dict


def getEntityFromForeignFieldType(field_type: str, entities_by_name: dict) -> str:
    if "<" not in field_type:
        return field_type

    type = field_type.split("<")[1].split(">")[0].strip()

    for entity_name in entities_by_name:
        if entity_name == type:
            return entity_name

    return ""


def is_unique_foreign_dto(dto_list: list, field_type: str) -> bool:
    for dto_type in dto_list:
        if dto_type == field_type:
            return True

    return False


def is_list_foreign_dto(dto_list: list, field_type: str) -> bool:
    if "<" not in field_type:
        return False

    type = field_type.split("<")[1].split(">")[0].strip()

    for dto_type in dto_list:
        if dto_type == type:
            return True

    return False


def generate_dto(
    root_path: str,
    template,
    dto_type,
    dto_data,
    dto_common_cmake_folder_path,
    default_init_values,
    application_cpp_domain_name,
    files_to_be_generated,
):
    fields = dto_data.get("fields", [])

    # only keep the fields from the current entity for initialization
    fields_init_values = ", ".join(
        [
            f"m_{field['name']}({default_init_values.get(field['type'], '{}')})"
            for field in fields
            if not field["is_foreign"]
        ]
    )

    # Get the headers to include based on the field types
    headers = dto_data["header_list"]

    dto_file_path = os.path.join(
        dto_common_cmake_folder_path,
        stringcase.snakecase(dto_data["feature_name"] + "_dto"),
        stringcase.snakecase(dto_data["feature_name"]),
        dto_data["file_name"],
    )

    if not files_to_be_generated.get(dto_file_path, False):
        return None

    dto_file_path = os.path.join(root_path, dto_file_path)

    # Create the directory if it does not exist
    os.makedirs(os.path.dirname(dto_file_path), exist_ok=True)

    with open(dto_file_path, "w") as f:
        f.write(
            template.render(
                feature_pascal_name=dto_data["feature_name"],
                dto_pascal_type=dto_type,
                fields=fields,
                is_relation_dto=dto_data.get("is_relation_dto", False),
                relation_fields=dto_data.get("relation_fields", []),
                headers=headers,
                fields_init_values=fields_init_values,
                application_cpp_domain_name=application_cpp_domain_name,
            )
        )
        print(f"Successfully wrote file {dto_file_path}")

    return dto_file_path


def generate_dto_files(
    root_path: str,
    manifest_file: str,
    files_to_be_generated: dict[str, bool] = None,
    uncrustify_config_file: str = None,
):
    with open(manifest_file, "r") as stream:
        try:
            manifest_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
            return

    dto_data = manifest_data.get("DTOs", [])
    application_name = manifest_data.get("global", {}).get(
        "application_name", "example"
    )
    application_name = stringcase.spinalcase(application_name)
    dto_common_cmake_folder_path = dto_data.get("common_cmake_folder_path", "")

    application_data = manifest_data.get("application", [])
    feature_list = application_data.get("features", [])

    global_data = manifest_data.get("global", [])
    application_cpp_domain_name = global_data.get(
        "application_cpp_domain_name", "Undefined"
    )

    # Organize feature_list by name for easier lookup
    feature_by_name = {feature["name"]: feature for feature in feature_list}

    template_env = Environment(loader=FileSystemLoader("templates/DTOs"))
    template = template_env.get_template("dto_template.h.jinja2")

    entities_data = manifest_data.get("entities", [])
    entities_list = entities_data.get("list", [])

    # Organize entities by name for easier lookup
    entities_by_name = {entity["name"]: entity for entity in entities_list}

    # Default initialization values
    default_init_values = {
        "int": "0",
        "float": "0.0f",
        "double": "0.0",
        "bool": "false",
        "QString": "QString()",
        "QDateTime": "QDateTime()",
        "QUuid": "QUuid()",
        "QUrl": "QUrl()",
        "QObject": "nullptr",
        "QList": "QList<>()",
        "RelationField": "RelationField::Undefined",
    }

    dto_dict, feature_ordered_dict = get_dto_dict_and_feature_ordered_dict(
        feature_by_name, entities_by_name
    )

    for feature_name, feature_data in feature_ordered_dict.items():
        generated_files = []
        feature_snake_name = stringcase.snakecase(feature_name)

        for dto_type, dto_data in feature_data["dtos"].items():
            generated_files.append(
                generate_dto(
                    root_path,
                    template,
                    dto_type,
                    dto_data,
                    dto_common_cmake_folder_path,
                    default_init_values,
                    application_cpp_domain_name,
                    files_to_be_generated,
                )
            )

        # strip generated files of None values
        generated_files = [file for file in generated_files if file]

        # generate these DTO's cmakelists.txt
        dto_cmakelists_template = template_env.get_template(
            "cmakelists_template.jinja2"
        )

        relative_dto_cmakelists_file = os.path.join(
            dto_common_cmake_folder_path,
            feature_snake_name + "_dto",
            "CMakeLists.txt",
        )
        dto_cmakelists_file = os.path.join(
            root_path,
            dto_common_cmake_folder_path,
            feature_snake_name + "_dto",
            "CMakeLists.txt",
        )

        # get all dto files, even those not generated
        dto_files = []
        for dto_type, dto_data in feature_data["dtos"].items():
            dto_files.append(
                os.path.join(
                    root_path,
                    dto_common_cmake_folder_path,
                    feature_snake_name + "_dto",
                    feature_snake_name,
                    dto_data["file_name"],
                )
            )

        ## Convert the file path to be relative to the directory of the cmakelists
        relative_dto_files = []
        for file_path in dto_files:
            relative_generated_file = os.path.relpath(
                file_path, os.path.dirname(dto_cmakelists_file)
            )
            relative_dto_files.append(relative_generated_file.replace("\\", "/"))

        # Get the feature dependencies and make them spinal case
        spinal_case_feature_dependencies = []
        for feature_dependency in feature_data["feature_dependencies"]:
            spinal_case_feature_dependencies.append(
                stringcase.spinalcase(feature_dependency)
            )

        rendered_template = dto_cmakelists_template.render(
            feature_spinal_name=stringcase.spinalcase(feature_snake_name),
            files=relative_dto_files,
            application_name=application_name,
            feature_dependencies=spinal_case_feature_dependencies,
        )

        # Create the directory if it does not exist
        os.makedirs(os.path.dirname(dto_cmakelists_file), exist_ok=True)

        if files_to_be_generated.get(relative_dto_cmakelists_file, False):
            with open(dto_cmakelists_file, "w") as fh:
                fh.write(rendered_template)
                print(f"Successfully wrote file {dto_cmakelists_file}")

    # generate common cmakelists.txt

    relative_dto_common_cmakelists_file = os.path.join(
        dto_common_cmake_folder_path, "CMakeLists.txt"
    )
    dto_common_cmakelists_file = os.path.join(
        root_path, relative_dto_common_cmakelists_file
    )
    if files_to_be_generated.get(relative_dto_common_cmakelists_file, False):
        ## After the loop, write the list of features folders to the common cmakelists.txt
        with open(dto_common_cmakelists_file, "w") as fh:
            for feature_name, _ in feature_ordered_dict.items():
                fh.write(
                    f"add_subdirectory({stringcase.snakecase(feature_name)}_dto)" + "\n"
                )
            print(f"Successfully wrote file {dto_common_cmakelists_file}")

    # format the files
    for file, to_be_generated in files_to_be_generated.items():
        # if uncrustify_config_file and files_to_be_generated.get(file, False):
        #     uncrustify.run_uncrustify(file, uncrustify_config_file)
        if to_be_generated and file.endswith(".h") or file.endswith(".cpp"):
            clang_format_runner.run_clang_format(os.path.join(root_path, file))


def get_files_to_be_generated(
    manifest_file: str, files_to_be_generated: dict[str, bool] = None
) -> list[str]:
    """
    Get the list of files that need to be generated based on the manifest file
    """
    # Read the manifest file
    with open(manifest_file, "r") as stream:
        try:
            manifest_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
            return

    dto_data = manifest_data.get("DTOs", [])
    dto_common_cmake_folder_path = dto_data.get("common_cmake_folder_path", "")

    application_data = manifest_data.get("application", [])
    feature_list = application_data.get("features", [])

    # Organize feature_list by name for easier lookup
    feature_by_name = {feature["name"]: feature for feature in feature_list}

    entities_data = manifest_data.get("entities", [])
    entities_list = entities_data.get("list", [])

    # Organize entities by name for easier lookup
    entities_by_name = {entity["name"]: entity for entity in entities_list}

    dto_dict, feature_ordered_dict = get_dto_dict_and_feature_ordered_dict(
        feature_by_name, entities_by_name
    )

    files = []
    for feature_name, feature_data in feature_ordered_dict.items():
        feature_snake_name = stringcase.snakecase(feature_name)

        for dto_type, dto_data in feature_data["dtos"].items():
            files.append(
                os.path.join(
                    dto_common_cmake_folder_path,
                    feature_snake_name + "_dto",
                    feature_snake_name,
                    dto_data["file_name"],
                )
            )

        dto_cmakelists_file = os.path.join(
            dto_common_cmake_folder_path,
            feature_snake_name + "_dto",
            "CMakeLists.txt",
        )

        files.append(dto_cmakelists_file)

    # generate common cmakelists.txt
    dto_cmakelists_file = os.path.join(dto_common_cmake_folder_path, "CMakeLists.txt")

    files.append(dto_cmakelists_file)

    # strip from files if the value in files_to_be_generated is False
    if files_to_be_generated:
        for path, generate in files_to_be_generated.items():
            if not generate and path in files:
                files.remove(path)

    return files


# generate the files into the preview folder
def preview_dto_files(
    root_path: str,
    manifest_file: str,
    files_to_be_generated: dict[str, bool] = None,
    uncrustify_config_file: str = None,
):
    manifest_preview_file = "temp/manifest_preview.yaml"

    # make a copy of the manifest file into temp/manifest_preview.yaml
    shutil.copy(manifest_file, manifest_preview_file)

    # modify the manifest file to generate the files into the preview folder
    with open(manifest_preview_file, "r") as fh:
        manifest = yaml.safe_load(fh)

    # remove .. from the path
    manifest["DTOs"]["common_cmake_folder_path"] = manifest["DTOs"][
        "common_cmake_folder_path"
    ].replace("..", "")

    # write the modified manifest file
    with open(manifest_preview_file, "w") as fh:
        yaml.dump(manifest, fh)

    root_path = os.path.join(root_path, "qleany_preview")

    #  remove .. from the path
    if files_to_be_generated:
        preview_files_to_be_generated = {}
        for path, value in files_to_be_generated.items():
            preview_files_to_be_generated[path.replace("..", "")] = value

        generate_dto_files(
            root_path,
            manifest_preview_file,
            preview_files_to_be_generated,
            uncrustify_config_file,
        )

    else:
        generate_dto_files(root_path, manifest_preview_file, {}, uncrustify_config_file)


# Main execution
if __name__ == "__main__":
    full_path = Path(__file__).resolve().parent

    # Add the current directory to the path so that we can import the generated files
    sys.path.append(full_path)

    # Set the current directory to the generator directory
    os.chdir(full_path)

    if len(sys.argv) > 1:
        manifest_arg = sys.argv[1]
        if manifest_arg.endswith(".yaml") or manifest_arg.endswith(".yml"):
            manifest_file = manifest_arg
            root_path = Path(manifest_file).parent

            if len(sys.argv) > 2 and sys.argv[2] == "--preview":
                preview_dto_files(root_path, manifest_file)
            else:
                # generate the files
                generate_dto_files(root_path, manifest_file)

        else:
            print("Error: Manifest file must be named 'qleany.yaml' or 'qleany.yml'")
    else:
        print("Error: Please provide the manifest file as an argument")



from jinja2 import Environment, FileSystemLoader
import yaml
import os
import sys
import stringcase
import shutil
import uncrustify
import clang_format_runner as clang_format_runner
from pathlib import Path

def generate_blank_qleany_yaml(folder_path: str):
    qleany_yaml = """
schema:
  version: 1
  
global:
  application_name: example
  application_cpp_domain_name: example
  organisation:
    name: example
    domain: example.com

entities:
  folder_path: src/entities/
  list:
    - name: Entity
      parent: EntityBase
      only_for_heritage: true
      fields:
        - type: QUuid
          name: uuid
        - type: QDateTime
          name: creationDate
        - type: QDateTime
          name: updateDate
    - name: FirstEntity
      parent: Entity
      fields:
        - type: QString
          name: content
        - type: SecondEntity
          name: secondEntity
          strong: true
        - type: QList<ThirdEntity>
          name: bucketOfThirdEntities
          ordered: true
          strong: true
    - name: SecondEntity
      parent: Entity
      fields:
        - type: QString
          name: name
    - name: ThirdEntity
      parent: Entity
      fields:
        - type: QString
          name: name
        - type: int
          name: age

repositories:
  list:
      - entity_name: FirstEntity
        lazy_loaders: true
      - entity_name: SecondEntity
        lazy_loaders: true
      - entity_name: ThirdEntity
        lazy_loaders: true
  repository_folder_path: src/persistence/repository/
  base_folder_path: src/persistence/ 

controller:
  folder_path: src/controller/
  create_undo_redo_controller: false

application:
  common_cmake_folder_path: src/application
  features:
    - name: FirstEntity
      DTO:
        dto_identical_to_entity:
          enabled: true
          entity_mappable_with: FirstEntity
      CRUD:
        enabled: true       
        entity_mappable_with: FirstEntity
        get:
          enabled: true
        get_all:
          enabled: true
        get_with_details:
          enabled: true
        create: 
          enabled: true
        remove: 
          enabled: true
        update:
          enabled: true
        insert_relation: 
          enabled: true
    - name: SecondEntity
      DTO:
        dto_identical_to_entity:
          enabled: true
          entity_mappable_with: SecondEntity
      CRUD:
        enabled: true       
        entity_mappable_with: SecondEntity
        get:
          enabled: true
        get_all:
          enabled: true
        get_with_details:
          enabled: false
        create: 
          enabled: true
        remove: 
          enabled: true
        update: 
          enabled: true
        change_active_status: 
          enabled: true
    - name: ThirdEntity
      DTO:
        dto_identical_to_entity:
          enabled: true
          entity_mappable_with: ThirdEntity
      CRUD:
        enabled: true       
        entity_mappable_with: ThirdEntity
        get:
          enabled: true
        get_all:
          enabled: true
        get_with_details:
          enabled: false
        create: 
          enabled: true
        remove: 
          enabled: true
        update: 
          enabled: true
        change_active_status: 
          enabled: true
    - name: Custom
      DTO:
        dto_identical_to_entity:
          enabled: false
      CRUD:
        enabled: false
      commands:
        - name: WriteRandomThings
          entities:
            - FirstEntity
            - SecondEntity
            - ThirdEntity
          validator: 
            enabled: True
          undo: False
          dto:
            in:
              type_prefix: WriteRandomThings
              fields:
                - type: QString
                  name: randomCarName
            out:
              enabled: false
        - name: RunLongOperation
          validator: 
            enabled: False
          undo: False
          minimum_progress_time: 1000
          dto:
            in:
              enabled: false
            out:
              enabled: false
        - name: CloseSystem
          entities:
            - FirstEntity
            - SecondEntity
            - ThirdEntity
          validator: 
            enabled: False
          undo: False
          dto:
            in:
              enabled: false
            out:
              enabled: false
      queries:
        - name: GetCurrentTime
          validator:
            enabled: false
          undo: False
          dto:
            in:
              enabled: false
            out:
              type_prefix: GetCurrentTimeReply
              fields:
                - type: QDateTime
                  name: currentDateTime

DTOs:
  common_cmake_folder_path: src/contracts.dto

CQRS:
  common_cmake_folder_path: src/contracts.cqrs

contracts:
  inverted_app_domain: com.example
  folder_path: src/contracts

presenter:
  folder_path: src/presenters
  create_undo_and_redo_singles: false
  singles:
    - name: SingleFirstEntity
      entity: FirstEntity
    - name: auto
      entity: SecondEntity
    - name: auto
      entity: ThirdEntity
  list_models:
    - name: ThirdEntityListModelFromFirstEntityBucketOfThirdEntities
      entity: ThirdEntity
      displayed_field: name
      in_relation_of: FirstEntity
      relation_field_name: bucketOfThirdEntities
    - name: FirstEntityListModel
      entity: FirstEntity  
      displayed_field: content

front_ends:
  qt_widgets:
    folder_path: src/qt_widgets_application/
"""

    with open(os.path.join(folder_path, "qleany.yaml"), "w") as file:
        file.write(qleany_yaml)